/**************************************************************************************************************
*
*       This file,
*
*                            /BasicIO.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _BasicIO_h
#define _BasicIO_h

#include "stringio.h"

#include <stdio.h>

#include <iostream>
#include <map>
#include <list>
#include <utility>
#include <string>

#define BasicIO_string_size	2000

using namespace std;

class BasicIO
{
	typedef void (BasicIO::*writeFunctionPtr)(FILE *);
	bool write(char *file_name, writeFunctionPtr pWriteFunction);

	public:

		enum Type_e
		{
			type_bool,
			type_int,
			type_uint,
			type_float,
			type_double,
			type_angle,
			type_string,
			type_file,
			type_configuration,
			type_directory,
#ifdef McArtim
			type_profile,
#endif
			type_enum
		};

		struct Parameter_s
		{
			// ~Parameter_s(){ cout << "destroying " << name_list.front() << '\n';}
			void *ptr;

			enum Type_e type;

			list<string> name_list;

			list< pair<int, string> > enum_map;

			const char *description;
			const char *enum_name_list_string;
			const char *name_list_string;
		};

	protected:

		const char *IOName;

		list<struct Parameter_s> ParameterList;

		void formatFilePath(char *file_path, char *file_name);

		char dir_base[BasicIO_string_size];
		char conf_file_name[BasicIO_string_size];

		void addParameter(void *ptr, const enum Type_e& t, const char *names, const char *enum_names, const char *description);

	public:

		BasicIO();

		inline const char *getIOName(){return IOName;}
		inline char *getConfFileName(){return conf_file_name;}

		static int getSizeof(enum Type_e t);

		bool read(char *file_name, bool cancel_on_errors=true);
		inline bool reload(){return read(conf_file_name);}

		void write(FILE *fptr);
		inline bool write(char *file_name){return write(file_name, &BasicIO::write);}

		void writeTemplate(FILE *fptr);
		inline bool writeTemplate(char *file_name){return write(file_name, &BasicIO::writeTemplate);}

		struct Parameter_s *findParameter(void *ptr);
		struct Parameter_s *findParameter(const string& name);

		bool O2I(const char *name, const char *value_string);
		bool I2O(struct Parameter_s *p, const char *value_string);
		bool I2O(void *ptr, const char *value_string);


		const char *getIOName(void *ptr);
		const char *getDescription(void *ptr);
		inline char *getBaseDirectory(){return dir_base;}
		inline void getBaseDirectoryFromFilename(char *file_name){extractPathFromFilename(file_name, dir_base);}
		
		virtual bool loadFiles(){return true;}
		bool readAndLoadFiles(char *conf_file_name);

};

#endif// _BasicIO_h


