/**************************************************************************************************************
*
*       This file,
*
*                            /BoxDetector.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _BoxDetector_h
#define _BoxDetector_h

#include <stdio.h>

#include "MacroCell.h"
#include "coordinate_sampling.h"
#include "SimulationObject.h"


class BoxDetector: public SimulationObject
{

/*
	for (monochromatic) actinic fluxes
*/

public:	// type declarations

	struct SamplingStructure_s
	{
		// sampling constants
		double x1, x2, Nx, cx, dx;
		double y1, y2, Ny, cy, dy;
		double z1, z2, Nz, cz, dz;

		// sampling functions
		double (*takeXSample)(const double& Nx, const double& cx);
		double (*takeYSample)(const double& Ny, const double& cy);
		double (*takeZSample)(const double& Nz, const double& cz);

		// volumes / areas
		double size[8];
	};

	enum SamplingElement_e
	{
		fullbox,
		boxface,
		boxtopface,
		boxbottomface,
		boxnorthface,
		boxsouthface,
		boxeastface,
		boxwestface
	};

public:

	bool assignDefinition(char *box_definition);
	bool takeEmissionSample(class Vector3D *pPosition, class Vector3D *pDirection);
	
	double getArea();

	static void writeDefinitionTemplate(FILE *fptr);

	static const char *getTypeString(enum SamplingElement_e be);
	// static const char *getFaceEmissivityString(enum FaceEmissivity_e fe);
	
	inline void enableCartesicGrid(bool enable=true){cartesic_grid=enable;}

protected:

	bool b_invert_faces;
	void sampleDirection(class Vector3D& direction);

	struct SamplingStructure_s SamplingStructure;

	enum SamplingElement_e SamplingElement;
	
	class MacroCell EmissionCell;
	
	bool cartesic_grid;

};

#endif




