/**************************************************************************************************************
*
*       This file,
*
*                            /Camera.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _Camera_h
#define _Camera_h

#include "Simulation.h"
#include "BasicIO.h"

struct DoubleRGB_s
{
	double r, g, b;
	class Vector3D direction;
};

struct RGBSensitivity_s
{
	double wavelength_nm;
	struct DoubleRGB_s DoubleRGB;
};

struct RGB_s
{
	unsigned char b;
	unsigned char g;
	unsigned char r;
	unsigned char alpha;
};

class Camera: public BasicIO
{
	class Simulation *pSimulation;
	class Geometry *pGeometry;
	class Raytracer *pRaytracer;
	
	struct DoubleRGB_s *DoubleRGB;
	struct RGBSensitivity_s *RGBSensitivity;
	int spectral_resolution;
	struct RGB_s *RGB;

	bool allocateScreen();
	bool loadSensitivityFunctions();

	bool calculateScreenPixelDirections();
	
	// IO ###############################################
		int xres, yres;		// resolution
		char fn_camera_rgb_sensitivity[BasicIO_string_size];
		char fn_simulation[BasicIO_string_size];
		int no_quadrature_trajectories;
		char fn_outfile[BasicIO_string_size];
		char fn_outbase[BasicIO_string_size];
		bool b_logarithmic_saturation;
		bool b_output_auxiliary_files;
	// end of IO ########################################
	
	void setupBasicIO();
	Camera(){};
public:

	Camera(class Simulation *SimulationPtr);
	~Camera();

	bool setup();
	bool run(const enum Simulation::RTESolvingMethod_e& method);
	bool writeResult();

};

#endif // _Camera_h


