/**************************************************************************************************************
*
*       This file,
*
*                            /Depiction/EPSFile.cpp,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#include "EPSFile.h"

EPSFile::EPSFile()
{
	fptr=(FILE *)0;
}

EPSFile::~EPSFile()
{
	close();
}

void EPSFile::beginDefinition(FILE *fptr, const char *name)
{
	fprintf(fptr,"/%s {\n",name);
}

void EPSFile::endDefinitionBind(FILE *fptr)
{
	fprintf(fptr,"} bind def\n\n");
}

void EPSFile::endDefinition(FILE *fptr)
{
	fprintf(fptr,"} def\n");
}

void EPSFile::defineParameter(FILE *fptr, const char *variable)
{
	fprintf(fptr,"\t/%s exch def\n",variable);
}

void EPSFile::defineFilledColoredCircle(FILE *fptr)
{
	beginDefinition(fptr,"FilledColoredCircle");
	defineParameter(fptr,"bval");
	defineParameter(fptr,"gval");
	defineParameter(fptr,"rval");
	defineParameter(fptr,"radius");
	defineParameter(fptr,"ypos");
	defineParameter(fptr,"xpos");
	fprintf(fptr,"\trval gval bval setrgbcolor\n");
	fprintf(fptr,"\txpos ypos radius 0 360 arc fill\n");
	endDefinitionBind(fptr);
}

void EPSFile::defineColoredCircle(FILE *fptr)
{
	beginDefinition(fptr,"ColoredCircle");
	defineParameter(fptr,"bval");
	defineParameter(fptr,"gval");
	defineParameter(fptr,"rval");
	defineParameter(fptr,"radius");
	defineParameter(fptr,"ypos");
	defineParameter(fptr,"xpos");
	fprintf(fptr,"\trval gval bval setrgbcolor\n");
	fprintf(fptr,"\txpos ypos radius 0 360 arc stroke\n");
	endDefinitionBind(fptr);
}

void EPSFile::defineFilledCircle(FILE *fptr)
{
	beginDefinition(fptr,"FilledCircle");
	defineParameter(fptr,"radius");
	defineParameter(fptr,"ypos");
	defineParameter(fptr,"xpos");
	fprintf(fptr,"\txpos ypos radius 0 360 arc fill\n");
	endDefinitionBind(fptr);
}

void EPSFile::defineCircle(FILE *fptr)
{
	beginDefinition(fptr,"Circle");
	defineParameter(fptr,"radius");
	defineParameter(fptr,"ypos");
	defineParameter(fptr,"xpos");
	fprintf(fptr,"\trval gval bval setrgbcolor\n");
	fprintf(fptr,"\txpos ypos radius 0 360 arc stroke\n");
	endDefinitionBind(fptr);
}

void EPSFile::defineFilledColoredRectangle(FILE *fptr)
{
	beginDefinition(fptr,"FilledColoredRectangle");
	defineParameter(fptr,"bval");
	defineParameter(fptr,"gval");
	defineParameter(fptr,"rval");
	defineParameter(fptr,"y2");
	defineParameter(fptr,"x2");
	defineParameter(fptr,"y1");
	defineParameter(fptr,"x1");
	fprintf(fptr,"\tnewpath\n");
	fprintf(fptr,"\tx1 y1 moveto\n");
	fprintf(fptr,"\tx2 y1 lineto\n");
	fprintf(fptr,"\tx2 y2 lineto\n");
	fprintf(fptr,"\tx1 y2 lineto\n");
	//fprintf(fptr,"%f %f lineto\n",x1,y1);
	fprintf(fptr,"\tclosepath\n");
	fprintf(fptr,"\trval gval bval setrgbcolor\n");
	fprintf(fptr,"\tfill\n");
	endDefinitionBind(fptr);
}

void EPSFile::defineColoredRectangle(FILE *fptr)
{
	beginDefinition(fptr,"ColoredRectangle");
	defineParameter(fptr,"bval");
	defineParameter(fptr,"gval");
	defineParameter(fptr,"rval");
	defineParameter(fptr,"y2");
	defineParameter(fptr,"x2");
	defineParameter(fptr,"y1");
	defineParameter(fptr,"x1");
	fprintf(fptr,"\tnewpath\n");
	fprintf(fptr,"\tx1 y1 moveto\n");
	fprintf(fptr,"\tx2 y1 lineto\n");
	fprintf(fptr,"\tx2 y2 lineto\n");
	fprintf(fptr,"\tx1 y2 lineto\n");
	//fprintf(fptr,"%f %f lineto\n",x1,y1);
	fprintf(fptr,"\tclosepath\n");
	fprintf(fptr,"\trval gval bval setrgbcolor\n");
	fprintf(fptr,"\tstroke\n");
	endDefinitionBind(fptr);
}

void EPSFile::defineFilledRectangle(FILE *fptr)
{
	beginDefinition(fptr,"FilledRectangle");
	defineParameter(fptr,"y2");
	defineParameter(fptr,"x2");
	defineParameter(fptr,"y1");
	defineParameter(fptr,"x1");
	fprintf(fptr,"\tnewpath\n");
	fprintf(fptr,"\tx1 y1 moveto\n");
	fprintf(fptr,"\tx2 y1 lineto\n");
	fprintf(fptr,"\tx2 y2 lineto\n");
	fprintf(fptr,"\tx1 y2 lineto\n");
	//fprintf(fptr,"%f %f lineto\n",x1,y1);
	fprintf(fptr,"\tclosepath\n");
	fprintf(fptr,"\tfill\n");
	endDefinitionBind(fptr);
}

void EPSFile::defineRectangle(FILE *fptr)
{
	beginDefinition(fptr,"Rectangle");
	defineParameter(fptr,"y2");
	defineParameter(fptr,"x2");
	defineParameter(fptr,"y1");
	defineParameter(fptr,"x1");
	fprintf(fptr,"\tnewpath\n");
	fprintf(fptr,"\tx1 y1 moveto\n");
	fprintf(fptr,"\tx2 y1 lineto\n");
	fprintf(fptr,"\tx2 y2 lineto\n");
	fprintf(fptr,"\tx1 y2 lineto\n");
	//fprintf(fptr,"%f %f lineto\n",x1,y1);
	fprintf(fptr,"\tclosepath\n");
	fprintf(fptr,"\tstroke\n");
	endDefinitionBind(fptr);
}

void EPSFile::writeDictionary(FILE *fptr)
{
	if(!fptr) return;
	fprintf(fptr,"\n/sizedpixelsize 0.4 def\n\n");
	// 
	defineFilledColoredCircle(fptr);
	defineColoredCircle(fptr);
	defineFilledCircle(fptr);
	defineCircle(fptr);
	//
	defineFilledColoredRectangle(fptr);
	defineColoredRectangle(fptr);
	defineFilledRectangle(fptr);
	defineRectangle(fptr);

}

bool EPSFile::open(char *filename, bool writeheader)
{
	close();
	if(!(fptr=fopen(filename,"w")))
		return false;
	if(writeheader)
	{
		fprintf(fptr,"%%!PS-Adobe-3.0 EPSF-3.0\n");
		fprintf(fptr,"%%%%BoundingBox: %d %d %d %d\n", Header.bb_x1, Header.bb_y1, Header.bb_x2, Header.bb_y2);
		fprintf(fptr,"%%%%Creator: %s\n", Header.creator);
		fprintf(fptr,"%%%%CreationDate: %s\n", Header.date);
		fprintf(fptr,"%%%%EndComments\n");
		fprintf(fptr,"%%%%EndProlog\n");
		writeDictionary(fptr);
	}
	return true;
}

void EPSFile::setScale(double xscale, double yscale)
{
	fprintf(fptr,"%f %f scale\n",xscale,yscale);
}

void EPSFile::close()
{
	if(fptr)
	{
		fprintf(fptr,"%%%%EOF\n");
		fclose(fptr);
		fptr=(FILE *)0;
	}
}

void EPSFile::setRGBColor(double r, double g, double b)
{
	fprintf(fptr,"%f %f %f setrgbcolor\n",r,g,b);
}

void EPSFile::rectangle(double x1,double y1,double x2,double y2)
{
	fprintf(fptr,"%f %f %f %f Rectangle\n",x1,y1,x2,y2);
}

void EPSFile::filledRectangle(double x1, double y1, double x2, double y2, double r, double g, double b)
{
	fprintf(fptr,"%f %f %f %f %f %f %f FilledColoredRectangle\n",x1,y1,x2,y2,r,g,b);
}

void EPSFile::putSizedPixel(double x, double y, double r, double g, double b, double size)
{
	fprintf(fptr,"%f %f %f %f %f %f FilledColoredCircle\n",x,y,size,r,g,b);
}

void EPSFile::putPixel(double x, double y, double r, double g, double b)
{
	fprintf(fptr,"%f %f sizedpixelsize %f %f %f FilledColoredCircle\n",x,y,r,g,b);
}

void EPSFile::greyImage(double x, double y, int xres, int yres, double *data)
{
	fprintf(fptr,"gsave\n");
	fprintf(fptr,"%lg %lg translate\n", x, y);
	fprintf(fptr,"%lg %lg scale\n", double(xres), double(yres));
	fprintf(fptr,"%d %d 8\t[%d 0 0 -%d 0 %d]\n", xres, yres, xres, yres, yres);
	fprintf(fptr,"{currentfile 3 %d mul string readhexstring pop} bind image\n", xres);
	int i,n=xres*yres;
	for(i=0;i<n;i++)
	{
		fprintf(fptr, "%.2x", (unsigned int)(256.0**data++)%256);
		if(!((i+1)%12))
			fputc('\n', fptr);
	}
	fprintf(fptr,"\ngrestore\n");
}

void EPSFile::colorImage(double x, double y, int xres, int yres, double *rdata, double *gdata, double *bdata)
{
	fprintf(fptr,"gsave\n");
	fprintf(fptr,"%lg %lg translate\n", x, y);
	fprintf(fptr,"%lg %lg scale\n", double(xres), double(yres));
	fprintf(fptr,"%d %d 8\t[%d 0 0 -%d 0 %d]\n", xres, yres, xres, yres, yres);
	fprintf(fptr,"{currentfile 3 %d mul string readhexstring pop} bind\n", xres);
	fprintf(fptr,"false 3 colorimage\n");
	int i,n=xres*yres;
	unsigned int r,g,b;
	for(i=0;i<n;i++)
	{
		r=(unsigned int)(256.0**rdata++)%256;
		g=(unsigned int)(256.0**gdata++)%256;
		b=(unsigned int)(256.0**bdata++)%256;
		fprintf(fptr, "%.6x",(unsigned int)((r<<16)|(g<<8)|b));
		if(!((i+1)%12))
			fputc('\n', fptr);
	}
	fprintf(fptr,"\ngrestore\n");
}

void EPSFile::colorImage(double x, double y, int xres, int yres, unsigned int *colordata)
{
	fprintf(fptr,"gsave\n");
	fprintf(fptr,"%lg %lg translate\n", x, y);
	fprintf(fptr,"%lg %lg scale\n", double(xres), double(yres));
	fprintf(fptr,"%d %d 8\t[%d 0 0 -%d 0 %d]\n", xres, yres, xres, yres, yres);
	fprintf(fptr,"{currentfile 3 %d mul string readhexstring pop} bind\n", xres);
	fprintf(fptr,"false 3 colorimage\n");
	int i,n=xres*yres;
	for(i=0;i<n;i++)
	{
		fprintf(fptr, "%.6x",*colordata++);
		if(!((i+1)%12))
			fputc('\n', fptr);
	}
	fprintf(fptr,"\ngrestore\n");
}

void EPSFile::selectFont(char *fontname, int fontsize)
{
	fprintf(fptr,"/%s\t%d\tselectfont\n", fontname, fontsize);
}

void EPSFile::putText(double x, double y, char *text)
{
	fprintf(fptr,"%lg\t%lg\tmoveto\t(%s)\tshow\n",x,y,text);
}

void EPSFile::line(double x1, double y1, double x2, double y2, double  linewidth)
{
	fprintf(fptr,"newpath\t");
	fprintf(fptr,"%lg %lg moveto\t%lg %lg lineto\t %lg setlinewidth\t stroke\n",x1,y1,x2,y2,linewidth);
}

//#define test_EPSFile
#ifdef test_EPSFile

#include <stdlib.h>

int main(int argc,char *argv[])
{
	EPSFile screen;
	char *testfilename="pss_test.ps";
	if(!screen.open(testfilename))
	{
		fprintf(stderr,"could not create file \"%s\"...\n",testfilename);
		return -1;
	}
	int x1=100,y1=100,x2=300,y2=400,i;
	screen.filledRectangle((double)x1,(double)y1,(double)x2,(double)y2,0.0,0.0,0.0);
	for(i=0;i<100;i++)
	{
		double x=(double)(x1+rand()%(x2-x1));
		double y=(double)(y1+rand()%(y2-y1));
		double r=(double)(rand()%256)/256.0;
		double g=(double)(rand()%256)/256.0;
		double b=(double)(rand()%256)/256.0;
		screen.putPixel(x,y,r,g,b);
	}
	screen.close();
	return 0;
}

#endif

