/**************************************************************************************************************
*
*       This file,
*
*                            /Depiction/EPSFile.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _EPSFile_h
#define _EPSFile_h

#include <stdio.h>

class EPSFile
{
public:

	struct Header_s
	{
		// bounding box
		int bb_x1, bb_y1;
		int bb_x2, bb_y2;
		// text information
		char creator[50];
		char date[100];
	};

private:
	FILE *fptr;
	struct Header_s Header;
	void writeDictionary(FILE *fptr);

	static void beginDefinition(FILE *fptr, const char *name);
	static void endDefinitionBind(FILE *fptr);
	static void endDefinition(FILE *fptr);
	static void defineParameter(FILE *fptr, const char *variable);
	static void defineFilledColoredCircle(FILE *fptr);
	static void defineColoredCircle(FILE *fptr);
	static void defineFilledCircle(FILE *fptr);
	static void defineCircle(FILE *fptr);
	static void defineFilledColoredRectangle(FILE *fptr);
	static void defineColoredRectangle(FILE *fptr);
	static void defineFilledRectangle(FILE *fptr);
	static void defineRectangle(FILE *fptr);
	/*static void defineGreyImage(FILE *fptr);
	static void defineColorImage(FILE *fptr);*/

public:

	EPSFile();
	~EPSFile();

	bool open(char *filename, bool writeheader=true);
	inline struct Header_s *getHeaderPtr(){return &Header;}
	void close();
	
	void setScale(double xscale, double yscale);
	void setRGBColor(double r, double g, double b);
	void rectangle(double x1, double y1, double x2, double y2);
	void filledRectangle(double x1, double y1, double x2, double y2, double r, double g, double b);
	void putSizedPixel(double x, double y, double r, double g, double b, double size);
	void putPixel(double x, double y, double r, double g, double b);

	void greyImage(double x, double y, int xres, int yres, double *data);
	void colorImage(double x, double y, int xres, int yres, double *rdata, double *gdata, double *bdata);
	// void colorImage(double x, double y, int xres, int yres, unsigned char *rdata, unsigned char *gdata, unsigned char *bdata);
	void colorImage(double x, double y, int xres, int yres, unsigned int *colordata);
	void selectFont(char *fontname, int fontsize);
	void putText(double x, double y, char *text);
	void line(double x1, double y1, double x2, double y2, double linewidth);
};

#endif //_EPSFile_h
