/**************************************************************************************************************
*
*       This file,
*
*                            /Depiction/RenderScreen.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _RenderScreen_h
#define _RenderScreen_h

#include "PointCollection.h"

class RenderScreen
{
public:
	struct Perspective_s
	{
		double distance;
		double factor;
		class Vector3D vector[4];
	};

private:
/*
	class for depiction of 3D points
*/
	int xres, yres;
	int x0, y0;
	unsigned int *data, background_color;	// 32 Bit Pixel
	
	double *z_buffer, *weight_buffer;		// depth information

	unsigned int **pixel;	// 2D pixel array

	// perspective:
	struct Perspective_s Perspective;

	// stereo stuff
	bool stereo_mode;
	double half_eye_distance;

public:
	RenderScreen();
	~RenderScreen();

	bool allocate(int xres, int yres, bool use_z_buffer=false);

	void toScreenPoint(class Vector3D& p, int& x, int& y);
	void toScreenPoint(class Vector3D& p, double& x, double& y);
	void absoluteToScreenPoint(class Vector3D p, int &x, int &y, double& depth);

	void clearScreen();
	bool renderPointCollection(class PointCollection *pPC);

	// perspective functions
	bool savePerspective(char *file_name);
	bool loadPerspective(char *file_name);

	// EPSFile functions...
	bool exportToEPSFile(class PointCollection *pPC, char *filename, bool export_image=true);

	// inline functions


	inline struct Perspective_s *getPerspectivePtr(){return &Perspective;}

	inline void defineFocusPoint(int x0, int y0){this->x0=x0;this->y0=y0;}
	inline bool isValid(){return data!=0 && pixel!=0;}

	inline unsigned int *getDataPtr(){return data;}
	inline double *getZBufferPtr(){return z_buffer;}
	inline double *getWeightBufferPtr(){return weight_buffer;}

	inline double getPerspectiveDistance(){return Perspective.distance;}
	inline double getPerspectiveFactor(){return Perspective.factor;}
	inline void setPerspective(double d, double fac){Perspective.distance=d;Perspective.factor=fac;}
	inline int getXRes(){return xres;}
	inline int getYRes(){return yres;}

	inline void setBackgroundColor(unsigned int bc){background_color=bc;}

	inline void enableStereoMode(bool enable=true){stereo_mode=enable;}
	inline bool hasStereoMode(){return stereo_mode;}
	inline void toggleStereoMode(){stereo_mode=stereo_mode?false:true;}

	inline void setHalfEyeDistance(double hed){half_eye_distance=hed;}
	inline double getHalfEyeDistance(){return half_eye_distance;}


};

#endif //_RenderScreen_h






