/**************************************************************************************************************
*
*       This file,
*
*                            /Depiction/TraceEventCollection.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _TraceEventCollection_h 
#define _TraceEventCollection_h

#include "PointCollection.h"

class RenderScreen;

class TraceEventCollection: public PointCollection
{
	char file_name[1024];
	unsigned int mapTypeToColor(int event_type, const double& contribution);

protected:

	int *event_type;			// event type of each point
	double *weight, *length, *contribution;
/*	double maximum_contribution;
	double minimum_contribution;
	double contribution_range;*/

	double contribution_center, contribution_sigma;

public:
	TraceEventCollection();
	~TraceEventCollection();

	/*inline double getColorScale(){return color_scale;}
	void setColorScale(const double& scale);*/

	inline double getContributionCenter(){return contribution_center;}
	inline void setContributionCenter(const double& a){contribution_center=a;}

	inline double getContributionSigma(){return contribution_sigma;}
	inline void setContributionSigma(const double& a){contribution_sigma=a;}

	void recalculateColors();
	
	void renderSensitivityTo(class RenderScreen *pRenderScreen);

	virtual bool allocatePoints(int n);
	virtual inline bool isValid(){return PointCollection::isValid() && event_type;}

	inline int *getEventTypeBase(){return event_type;}

	char *getTraceEventName(int index, char *buffer);

	bool loadFromFile(char *filename, bool *enable);
	inline bool reload(bool *enable){return loadFromFile(file_name, enable);}
};

#endif //_TraceEventCollection_h

