/**************************************************************************************************************
*
*       This file,
*
*                            /FunctionalLayerIO.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _FunctionalLayerIO_h
#define _FunctionalLayerIO_h

#include "BasicIO.h"

class FunctionalLayerIO: public BasicIO
{

public:
	FunctionalLayerIO();
	~FunctionalLayerIO();

	// information
	inline bool usePrecision(){return b_use_precision;}
	inline int getPrecisionCheckCount(){return precision_check_count;}

	inline char *getConstituentsPtr(){return constituents;}
	inline char *getPathIntegralsPtr(){return path_integrals;}

	inline void enableHessianCalculation(bool enable=true){b_calculate_Hessian=enable;}
	inline void enableEliminateAbsorption(bool enable=true){b_eliminate_absorption=enable;}
	inline void enableCorrectForOpticalFactors(bool enable=true){b_correct_for_optical_factors=enable;}


protected:

	// IO #######################################################
	char output_format[BasicIO_string_size];
	char constituents[BasicIO_string_size];
	char path_integrals[BasicIO_string_size];

	bool b_output_polarized_quantities;
	bool b_normalize_with_F0;

	bool b_sun_disc_sampling;			// sample from the sun disc
	bool b_use_precision;
	double precision;
	bool b_precision_major_limit;
	int precision_check_count;

	bool b_enable_variance_reduction;

	bool b_event_counts;
	bool b_eliminate_absorption;
	bool b_correct_for_optical_factors;
	bool b_calculate_Hessian;

	bool b_suppress_Hessian_output;
	bool b_suppress_Jacobian_output;

	// sensitivity grid file
	bool b_create_sensitivity_grid_file;
	char fn_sensitivity_grid_file [BasicIO_string_size];

	// LE contribution histogram
	bool b_create_LEC_histogram;
	int no_LEC_bins;
	double minimum_LEC, maximum_LEC, LEC_range;

	// end of IO ################################################

	class list<string> constituents_list, path_integrals_list, total_constituents_list;
	
};

#endif // _FunctionalLayerIO_h


