/**************************************************************************************************************
*
*       This file,
*
*                            /Grid1D.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _Grid1D_h
#define _Grid1D_h

#include <stdio.h>

class Grid1D
{
	double A,B, *g, *mg;
	int resolution;		// Attention:  there are resolution+1 grid values!
	char name[20], unit[5];

public:

	Grid1D();
	~Grid1D();

	inline char *getName(){return name;}
	bool setResolution(int n);

	bool loadFromFile(char *filename);
	bool loadFromFile(FILE *fptr);

	void printToFile(FILE *fptr);
	void printGridInfo(FILE *fptr, bool as_comment=false);

	void defineNameAndUnit(const char *name, const char *unit);

	bool checkGrid();

	inline double *getGridPtr(){return g;}
	inline double *getMiddleGridPtr(){return mg;}

	inline int getResolution(){return resolution;}

	inline double getA(){return A;}
	inline double getB(){return B;}

	inline void setA(double x){A=x;}
	inline void setB(double x){B=x;}
	
	inline double& operator[](const int& i){return g[i];}

	bool findIndex(double& x, int *pIndex);
	inline bool findIntervallIndexSlowButSafe(double& x, int *pIndex){return (x>=A && x<=B)?findIntervallIndexFastButUnsafe(x, pIndex):false;}
	bool findIntervallIndexFastButUnsafe(double& x, int *pIndex);
	
};

#endif //_LatticeGrid_h



