/**************************************************************************************************************
*
*       This file,
*
*                            /GridIO.cpp,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#include "GridIO.h"

double GridIO::planet_radius=6377.5;
double GridIO::ground_level=0.0;

GridIO::GridIO()
{
	IOName="grid";
	GridType=sp1D;

	ground_level=0.0;
	
	b_cyclic_x=false;
	b_cyclic_y=false;
	b_cyclic_z=false;

	b_s3d_one_longitude_cell=false;
	b_s3d_one_colatitude_cell=false;
	
	addParameter(&GridType, type_enum, "GridType|type", "sp1D|sp3D|pp1D|pp3D", "grid type sp = spherical, pp = plane parallel, 1D or 3D");		
	addParameter(&planet_radius, type_double, "planet_radius|Re|earth_radius", 0, "[km] radius of the planet for spherical grids");
	addParameter(&ground_level, type_double, "ground_level|ground_altitude", 0, "[km] ground height level");
	addParameter(fn_longitude_grid, type_file, "fn_longitude_grid|fn_phi_grid", 0, "[degree] 1 column");
	addParameter(fn_colatitude_grid, type_file, "fn_colatitude_grid|fn_theta", 0, "[degree] 1 column");
	addParameter(fn_altitude_grid, type_file, "fn_altitude_grid", 0, "[km] 1 column");
	addParameter(fn_X_grid, type_file, "fn_X_grid", 0, "[km] 1 column");
	addParameter(fn_Y_grid, type_file, "fn_Y_grid", 0, "[km] 1 column");
	addParameter(fn_Z_grid, type_file, "fn_Z_grid", 0, "[km] 1 column");
	addParameter(&b_cyclic_x, type_bool, "b_cyclic_x|b_cyclic_longitude", 0, "cyclic x / longitude coordinate?");
	addParameter(&b_cyclic_y, type_bool, "b_cyclic_y|b_cyclic_colatitude", 0, "cyclic y / colatitude coordinate?");
	addParameter(&b_cyclic_z, type_bool, "b_cyclic_z|b_cyclic_radius", 0, "cyclic z / radius coordinate?");
	addParameter(&b_s3d_one_longitude_cell, type_bool, "b_s3d_one_longitude_cell", 0, "only one huge longitude cell?");
	addParameter(&b_s3d_one_colatitude_cell, type_bool, "b_s3d_one_colatitude_cell", 0, "only one huge colatitude cell?");
}



