/**************************************************************************************************************
*
*       This file,
*
*                            /GridIO.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _GridIO_H
#define _GridIO_H

#include "BasicIO.h"

class GridIO: public BasicIO
{

public:
	enum GridType_e{sp1D,sp3D,pp1D,pp3D};	// grid type sp = spherical, pp = plane parallel
	GridIO();
	virtual ~GridIO(){}
	
	void formatAltitudeGridFilename(char *filepath){formatFilePath(filepath, fn_altitude_grid);}
	void formatColatitudeGridFilename(char *filepath){formatFilePath(filepath, fn_colatitude_grid);}
	void formatLongitudeGridFilename(char *filepath){formatFilePath(filepath, fn_longitude_grid);}
	
	void formatXGridFilename(char *filepath){formatFilePath(filepath,fn_X_grid);}
	void formatYGridFilename(char *filepath){formatFilePath(filepath,fn_Y_grid);}
	void formatZGridFilename(char *filepath){formatFilePath(filepath,fn_Z_grid);}

	inline static double getPlanetRadius(){return planet_radius;}
	inline static double getGroundLevel(){return ground_level;}

	inline enum GridType_e getGridType(){return GridType;}

protected:
	
	// grids and dimension ####################################################
	enum GridType_e GridType;	// grid type sp = spherical, pp = plane parallel
	
	// global paramaters
	static double planet_radius;		// radius of the planet for spherical grids
	static double ground_level;		// ground height level
		
	// spherical grids
	char fn_longitude_grid[BasicIO_string_size];		// X grid file
	char fn_colatitude_grid[BasicIO_string_size];		// Y grid file
	char fn_altitude_grid[BasicIO_string_size];		// Z grid file

	// cartesic grids
	char fn_X_grid[BasicIO_string_size];		// X grid file
	char fn_Y_grid[BasicIO_string_size];		// Y grid file
	char fn_Z_grid[BasicIO_string_size];		// Z grid file

	// cyclic coordinates
	bool b_cyclic_x;			// cyclic x/longitude coordinate
	bool b_cyclic_y;			// cyclic y/colatitude coordinate
	bool b_cyclic_z;			// cyclic z/height_coordinate

	// spherical 3D grid perculiarities
	bool b_s3d_one_longitude_cell;
	bool b_s3d_one_colatitude_cell;
};

#endif



