/**************************************************************************************************************
*
*       This file,
*
*                            /HITRAN/HITRANCalculator.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _HITRANCalculator_h
#define _HITRANCalculator_h

#include "HITRANDataBase.h"
#include "../Math/LinearelyInterpolatedFunctionTemplate.h"

struct HITRANAtmosphericGrid_s
{
	class LinearelyInterpolatedFunctionTemplate<double> pressure;
	class LinearelyInterpolatedFunctionTemplate<double> temperature;
	double *height;
	int no_layers;
	int no_wavelengths;
	double *data,**cs;
	double *nu,*wavelength;
};


enum ProfileModel_e
{
	PFM_constant_vmr,
	PFM_vmr_profile,	
	PFM_number_density_profile
};

class HITRANCalculator: public HITRANDataBase
{
	// IO ----------------------------------------
	char fn_pressure[BasicIO_string_size];			// filename of the molparam.txt
	char fn_temperature[BasicIO_string_size];		// filename of the molparam.txt
	char fn_height_grid[BasicIO_string_size];		// filename of the molparam.txt
	// end of IO ----------------------------------------

	// wavelength and height grid
	bool mapAndAllocateWavenumberGrid(double **pnu, double *wavelength, int no_wavelengths, double *pnu_min, double *pnu_max);
	struct HITRANAtmosphericGrid_s HITRANAtmosphericGrid;

	// HITRANAtmosphericGrid
	bool initAtmosphericGrid(int no_wavelengths);

public:

	HITRANCalculator();
	~HITRANCalculator();

	bool getCrossSection(char *MoleculeName, double vmr, double T, double p, double *wavelength, int no_wavelengths, double *cross_section);

	// inline struct HITRANAtmosphericGrid_s *getHITRANAtmosphericGridPtr(){return &HITRANAtmosphericGrid;}
	// bool processAtmosphericGrid(char *MoleculeName, double *wavelength, int no_wavelengths, enum ProfileModel_e ProfileModel, void *vmr_profile, bool compute_absorption_coefficient);

	// IO shortcuts
	inline void formatTemperatureFilename(char *filename){formatFilePath(filename,fn_temperature);}
	inline void formatPressureFilename(char *filename){formatFilePath(filename,fn_pressure);}
	inline void formatHeightGridFilename(char *filename){formatFilePath(filename,fn_height_grid);}


};

#endif // _HITRANCalculator_h




