/**************************************************************************************************************
*
*       This file,
*
*                            /HITRAN/HITRANDataBase.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _HITRANDataBase_h
#define _HITRANDataBase_h

#include "../BasicIO.h"
#include "HITRANMolecule.h"

class HITRANDataBase: public BasicIO
{
	friend class HITRANMolecule;
protected:

	// IO parameters
	char dir_molecule_directory[BasicIO_string_size];	// directory of the "by-molecule" folder
	char database_name[BasicIO_string_size];		// e.g. hit04
	char fn_molparam[BasicIO_string_size];			// filename of the molparam.txt
	char fn_parsum[BasicIO_string_size];			// file with partition function sums at T_ref

	/*double T_ref;					// reference temperature
	double p_ref;					// reference pressure
	double wing_cut_off;				// wing cut off*/

	bool suppress_warning_messages, database_initialized;


	// molecules
	class HITRANMolecule *molecule;
	int no_molecules, no_isotopes;
	bool readGlobalMoleculeParameters();
	bool hasLocalMoleculeDataSet(int molecule_index);
	void formatMoleculeParameterFile(int molecule_index, char *filename);

	// partition sum data
	double *parsum_data,*temperature,**partition_sum;
	double temperature_minimum, temperature_maximum;
	int no_temperatures;
	bool readPartitionSums();
	bool determinePartitionSumIndex(int *pPartitionIndex, int molecule_index, int isotope_index);
	double calculatePartitionSum(int partition_sum_index, double T);
	void compressPartitionSumLine();

	char isotopes_line[4000];
	char isotopes_line_buffer[4000];

public:

	HITRANDataBase();
	~HITRANDataBase();

	bool open(char *filename);
	bool getMoleculeIndex(char *MoleculeName, int *pMoleculeIndex);
	
	// inlines
	inline class HITRANMolecule *getMoleculePtr(){return molecule;}
	class HITRANMolecule *getMoleculePtr(char *MoleculeName);

	inline bool isInitialized(){return database_initialized;}
	inline void enableWarnings(bool enable=true){suppress_warning_messages=enable;}

	// IO shortcuts
	inline void formatGlobalMoleculeParameterFilename(char *filename){formatFilePath(filename,fn_molparam);}
	inline void formatPartitionSumFilename(char *filename){formatFilePath(filename,fn_parsum);}
};

#endif // _HITRANDataBase_h


