/**************************************************************************************************************
*
*       This file,
*
*                            /HITRAN/HITRANMolecule.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _HITRANMolecule_h
#define _HITRANMolecule_h

#include "LineParameterFile.h"
// #include "SpectralWindow.h"

struct HITRANMoleculeIsotope_s
{
	int iso;
	double abundance;
	double Q_ref;
	int gj;
	double molar_mass;	
	// non database variables
	int partition_sum_index;
};


class HITRANDataBase;

class HITRANMolecule: public LineParameterFile
{
	public:
		struct SpectralRange_s
		{
			// window
			double nuA_pcm, nuB_pcm;		// wavenumber window
			double lambdaA, lambdaB;		// wavelength window
			int iA, iB, no_lines;			// indices
		};

	private:
		// static variables...
		friend class HITRANDataBase;
		static class HITRANDataBase *pHITRANDataBase;

		double wing_cut_off;
		double T_ref, p_ref;
		bool suppress_warning_messages;

		struct SpectralRange_s SpectralRange;

		// specific variables
		char name[10];
		int HITRANIndex, molecule_index;

		// isotopes...
		int no_isotopes;
		struct HITRANMoleculeIsotope_s *HITRANMoleculeIsotope;

		// line parameter correction
		// void calculateIsotopePartitionSums();
		// void correctLineParameters(struct CorrectedLineParameterStruct_s *pCLPS, struct LineParameterStruct_s *pLPS);
	public:

		HITRANMolecule();
		~HITRANMolecule();

		bool load();
		bool allocateIsotopes(int n);
		bool extractRelevantLinesIn(const double& lambdaA, const double& lambdaB);
		// bool extractRelevantLines(class SpectralWindow *pSW);
		inline struct SpectralRange_s *getSpectralRangePtr(){return &SpectralRange;}

		// inlines
		inline void setReference(double T, double p){T_ref=T; p_ref=p;}
		inline double getReferenceTemperature(){return T_ref;}
		inline double getReferencePressure(){return p_ref;}

		inline void setWingCutOff(double wco){wing_cut_off=wco;};
		inline double getWingCutOff(){return wing_cut_off;}

		inline int getNOIsotopes(){return no_isotopes;}
		inline struct HITRANMoleculeIsotope_s *getIsotopePtr(){return HITRANMoleculeIsotope;}

		double calculatePartitionSum(struct HITRANMoleculeIsotope_s *pMI, const double& T);

		// static members...
		static inline void setHITRANDataBasePtr(class HITRANDataBase *HITRANDataBasePtr){pHITRANDataBase=HITRANDataBasePtr;}
};

#endif //HITRANMolecule




