/**************************************************************************************************************
*
*       This file,
*
*                            /HITRAN/LineParameterFile.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _LineParameterFile_h
#define _LineParameterFile_h

#include <string.h>

#define HITRANLineParameterRecordSize		162
#define HITRANLineParameters			19

typedef char Hollerith[15];

struct LineParameterStruct_s
{
	int M;
	int I;
	double nu;
	double S;
	double A;
	double gamma_air;
	double gamma_self;
	double E_dash_dash;
	double n_air;
	double delta_air;
	Hollerith V_dash;
	Hollerith V_dash_dash;
	Hollerith Q_dash;
	Hollerith Q_dash_dash;
	char I_err[6];
	char I_ref[12];
	char flag;
	double g_dash;
	double g_dash_dash;
};

enum HITRANLineParameter_e
{
	HLP_M,
	HLP_I,
	HLP_nu,
	HLP_S,
	HLP_A,
	HLP_gamma_air,
	HLP_gamma_self,
	HLP_E_dash_dash,
	HLP_n_air,
	HLP_delta_air,
	HLP_V_dash,
	HLP_V_dash_dash,
	HLP_Q_dash,
	HLP_Q_dash_dash,
	HLP_I_err,
	HLP_I_ref,
	HLP_flag,
	HLP_g_dash,
	HLP_g_dash_dash
};

class LineParameterFile
{
	enum LPType_e
	{
		LPType_real,
		LPType_integer,
		LPType_flag,
		LPType_string,
		LPType_Hollerith
	};

	struct LineParameter_s
	{
		int length, position;
		enum LPType_e type;
		int offset;
	};

	static struct LineParameterStruct_s DummyLPS;
	static struct LineParameter_s LineParameter[HITRANLineParameters];

	// enabled parameters
	struct LineParameter_s *pLineParameter[HITRANLineParameters];
	int no_enabled_parameters;
	bool parameterMask[HITRANLineParameters];

protected:
	char *block_data;
	struct LineParameterStruct_s *LineParameterStruct;
	int no_spectral_lines;

	char filename[2047];
	inline void setFilename(char *fn){strcpy(filename,fn);}

	bool loadBlockData(char *filename);


	void resetEnabledParameters();
	void enableDefaultParameters();
	void enableParameter(enum HITRANLineParameter_e HLP, bool enable=true){parameterMask[int(HLP)]=enable;}
	void createEnabledParametersList();

	bool extractLineParameters(int iA, int iB, bool parameterMask[HITRANLineParameters]);
	void extractLineParameters(int index, bool parameterMask[HITRANLineParameters]);

	// inlines
	inline int getNOSpectralLines(){return no_spectral_lines;}
	inline bool extractEnabledLineParameters(int iA, int iB){return extractLineParameters(iA,iB,parameterMask);}
	inline void extractEnabledLineParameters(int index){extractLineParameters(index,parameterMask);}
	inline bool extractEnabledLineParameters(){return extractLineParameters(0,no_spectral_lines-1,parameterMask);}

public:
	LineParameterFile();
	~LineParameterFile();

	inline struct LineParameterStruct_s *getLineParameterStructPtr(){return LineParameterStruct;}
};

#endif //_LineParameterFile_h
