/**************************************************************************************************************
*
*       This file,
*
*                            /HITRAN/SpectralWindow.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _SpectralWindow_h
#define _SpectralWindow_h

#include "../Math/Matrix.h"
#include "../Math/Vector.h"

#include "HITRANMolecule.h"

class SpectralWindow
{
		// static class HITRANDataBase *pHITRANDataBase;
		struct AtmosphericLineParameterStruct_s
		{
			double nu, nu_A, nu_B;		// line transition frequency			cm^-1]
			double S;			// spectral line intensity			cm^-1(molec*cm^-2)] @ T_ref
			double gamma_L;			// air-broadended halfwidth at half maximum	[cm^-1/atm] @ T_ref
			double gamma_D;			// doppler broadended half width
			double y;
			double Area;
		};

		struct AtmosphericLineParameterStruct_s *AtmosphericLPS;
		class Matrix AtmosphericLPSData;

		class HITRANMolecule *pHITRANMolecule;
		struct HITRANMolecule::SpectralRange_s *pSpectralRange;
		class Vector Q_T;
		
		int line_index;		// last line index
		int findLineIndex(const double& nu);

	protected:

		double p;			// pressure
		double p_s;			// partial pressure = vmr*p
		double T;			// temperature

	public:

		SpectralWindow();

		bool allocateSpectralRange(class HITRANMolecule *HITRANMoleculePtr);
		void calculateLineParameters(const double& p, const double& p_s, const double& T);

		double calculateCrossSection(const double& lambda);
};

#endif // _SpectralWindow_h




