/**************************************************************************************************************
*
*       This file,
*
*                            /HITRAN/Voigt.cpp,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#include <math.h>
#include "Voigt.h"

/*
double KuntzHumlicekVoigtProfile(double x, double y)
{
//	Voigt Profile Calculation from paper:
//	M. Kuntz:	"A New Implementation of the Humlicek Algorithm for the Calculation of the Voigt Profile Function"
	double test1=fabs(x)+y;
	double x2=x*x;
	double x4=x2*x2;
	double x6=x4*x2;
	double x8=x4*x4;
	double x10=x6*x4;
	double x12=x6*x6;
	double x14=x6*x8;
	double x16=x8*x8;
	double x18=x8*x10;
	double x20=x10*x10;
	double x22=x12*x10;
	double x24=x12*x12;
	double x26=x14*x12;
	double x28=x14*x14;
	double y2=y*y;
	double y3=y2*y;
	double y4=y3*y;
	double y5=y4*y;
	double y6=y5*y;
	double y7=y6*y;
	double y8=y7*y;
	double y9=y8*y;
	double y10=y9*y;
	double y11=y10*y;
	double y12=y11*y;
	double y13=y12*y;
	double y14=y13*y;
	double y15=y14*y;
	double y16=y15*y;
	double y17=y16*y;
	double y18=y17*y;
	double y19=y18*y;
	double y20=y19*y;
	double y21=y20*y;
	double y22=y21*y;
	double y23=y22*y;
	double y24=y23*y;
	double y25=y24*y;
	double y26=y25*y;
	double y27=y26*y;
	double y28=y27*y;
	if(test1>=15)
	{
		//region 1
		double a1=0.2820948*y+0.5641896*y3;
		double b1=0.5641896*y;
		double a2=0.25+y2+y4;
		double b2=-1.0+2*y2;
		return (a1+b1*x2)/(a2+b2*x2+x4);
	}
	else if(test1>=5.5)
	{
		//region 2
		double a3=1.05786*y+4.65456*y3+3.10304*y5+0.56419*y7;
		double b3=2.962*y+0.56419*y3+1.69257*y5;
		double c3=1.69257*y-2.53885*y3;
		double d3=0.56419*y;
		double a4=0.5625+4.5*y2+10.5*y4+6.0*y6+y8;
		double b4=-4.5+9.0*y2+6.0*y4+4.0*y6;
		double c4= 10.5 - 6.0*y2 + 6.0*y4;
		double d4= -6.0+4*y2;
		return (a3+b3*x2+c3*x4+d3*x6)/(a4+b4*x2+c4*x4+d4*x6+x8);
	}
	else if(y-0.195*fabs(x)>-0.176)
	{
		//region 3
		double a5 = 272.102 + 973.778*y + 1629.76*y2 + 1678.33*y3 + 1174.8*y4 + 581.746*y5 + 204.510*y6 + 49.5213*y7 + 7.55895*y8 + 0.564224*y9;
		double b5 = -60.5644 - 2.34403*y + 220.843*y2 + 336.364*y3 + 247.198*y4 + 100.705*y5 + 22.6778*y6 + 2.25689*y7;
		double c5 = 4.58029 + 18.546*y + 42.5683*y2 + 52.8454*y3 + 22.6798*y4 + 3.38534*y5;
		double d5 = -0.128922 + 1.66203*y + 7.56186*y2 + 2.25689*y3;
		double e5 = 0.000971457 + 0.564224*y;
		double a6 = 272.102 + 1280.83*y + 2802.87*y2 + 3764.97*y3 + 3447.63*y4 + 2256.98*y5 + 1074.41*y6 + 369.199*y7 + 88.2674*y8 + 13.3988*y9 + y10;
		double b6 = 211.678 + 902.306*y + 1758.34*y2 + 2037.31*y3 + 1549.68*y4 + 793.427*y5 + 266.299*y6 + 53.5952*y7 + 5.0*y8;
		double c6 = 78.866 + 308.186*y + 497.302*y2 + 479.258*y3 + 269.292*y4 + 80.3928*y5 + 10.0*y6;
		double d6 = 22.0353 + 55.0293*y + 92.7586*y2 + 53.5952*y3 + 10.0*y4;
		double e6 = 1.49645 + 13.3988*y + 5.0*y2;
		return (a5 + b5*x2 + c5*x4 + d5*x6 + e5*x8)/(a6 + b6*x2 + c6*x4 + d6*x6 + e6*x8 + x10);
	}
	else
	{
		//region 4
		double a7 = 1.16028e9*y - 9.86604e8*y3 + 4.56662e8*y5 - 1.53575e8*y7 + 4.08168e7*y9 - 9.69463e6*y11 + 1.6841e6*y13 - 320772.0*y15 + 40649.2*y17 - 5860.68*y19 + 571.687*y21 - 72.9359*y23 + 2.35944*y25 - 0.56419*y27;
		double b7 = -5.60505e8*y - 9.85386e8*y3 + 8.06985e8*y5 - 2.91876e8*y7 + 8.64829e7*y9 - 7.72359e6*y11 + 3.59915e6*y13 - 234417*y15 + 45251.3*y17 - 2269.19*y19 - 234.143*y21 + 23.0312*y23 - 7.33447*y25;
		double c7 = -6.51523e8*y + 2.47157e8*y3 + 2.94262e8*y5 - 2.04467e8*y7 + 2.29302e7*y9 - 2.3818e7*y11 + 576054*y13 + 98079.1*y15 - 25338.3*y17 + 1097.77*y19 + 97.6203*y21 - 44.0068*y23;
		double d7 = -2.63984e8*y + 2.70167e8*y3 - 9.96224e7*y5 - 4.15013e7*y7 + 3.83112e7*y9 + 2.2404e6*y11 - 303569*y13 - 66431.2*y15 + 8381.97*y17 + 228.563*y19 - 161.358*y21;
		double e7 = -6.31771e7*y + 1.40677e8*y3 + 5.56965e6*y5 + 2.46201e7*y7 + 468142*y9 - 1.003e6*y11 - 66212.1*y13 + 23507.6*y15 + 296.38*y17 - 403.396*y19;
		double f7 = - 1.69846e7*y + 4.07382e6*y3 - 3.32896e7*y5 - 1.93114e6*y7 - 934717*y9 + 8820.94*y11 + 37544.8*y13 + 125.591*y15 - 726.113*y17;
		double g7 = - 1.23165e6*y + 7.52883e6*y3 - 900010*y5 - 186682*y7+ 79902.5*y9+ 37371.9*y11- 260.198*y13 - 968.15*y15;
		double h7 = -610622*y + 86407.6*y3 + 153468*y5 + 72520.9*y7 + 23137.1*y9 - 571.645*y11 - 968.15*y13;
		double o7 = -23586.5*y + 49883.8*y3 + 26538.5*y5 + 8073.15*y7 - 575.164*y9 - 726.113*y11;
		double p7 = -8009.1*y + 2198.86*y3 + 953.655*y5 - 352.467*y7 - 403.396*y9;
		double q7 = -622.056*y - 271.202*y3 - 134.792*y5 - 161.358*y7;
		double r7 = -77.0535*y - 29.7896*y3 - 44.0068*y5;
		double s7 = -2.92264*y - 7.33447*y3;
		double t7 = -0.56419*y;
// ^ checked , v unchecked
		double a8 = 1.02827e9 - 1.5599e9*y2 +1.17022e9*y4 - 5.79099e8*y6 + 2.11107e8*y8 - 6.11148e7*y10 + 1.44647e7*y12 - 2.85721e6*y14 + 483737.0*y16 - 70946.1*y18 + 9504.65*y20 - 955.194*y22 + 126.532*y24 - 3.68288*y26 + y28;
		double b8 = 1.5599e9 - 2.28855e9*y2 + 1.66421e9*y4 - 7.53828e8*y6 + 2.89676e8*y8 - 7.01358e7*y10 + 1.39465e7*y12- 2.84954e6*y14 + 498334.0*y16 - 55600.0*y18 + 3058.26*y20 + 533.254*y22 - 40.5117*y24 + 14.0*y26;
		double c8 = 1.17022e9 - 1.66421e9*y2 + 1.06002e9*y4 - 6.60078e8*y6 + 6.33496e7*y8 - 4.60396e7*y10 + 1.4841e7*y12 - 1.06352e6*y14 - 217801.0*y16 + 48153.3*y18 - 1500.17*y20- 198.876*y22 + 91.0*y24;
		double d8 = 5.79099e8 - 7.53828e8*y2 + 6.60078e8*y4 + 5.40367e7*y6 + 1.99846e8*y8 - 6.87656e6*y10- 6.89002e6*y12 + 280428*y14 + 161461*y16 - 16493.7*y18 - 567.164*y20 + 364.0*y22;
		double e8 = 2.11107e8 - 2.89676e8*y2 + 6.33496e7*y4 - 1.99846e8*y6 - 5.01017e7*y8 - 5.25722e6*y10 + 1.9547e6*y12 + 240373.0*y14 - 55582.0*y16 - 1012.79*y18 + 1001.0*y20;
		double f8 = 6.11148e7 - 7.01358e7*y2 + 4.60396e7*y4 - 6.87656e6*y6 + 5.25722e6*y8 + 3.04316e6*y10 + 123052*y12 - 106663*y14 - 1093.82*y16 + 2002*y18;
		double g8 = 1.44647e7 - 1.39465e7*y2 + 1.4841e7*y4 + 6.89002e6*y6 + 1.9547e6*y8 - 123052.0*y10 - 131337.0*y12 - 486.14*y14 + 3003*y16;
		double h8 = 2.85721e6 - 2.84954e6*y2 + 1.06352e6*y4 + 280428*y6 - 240373.0*y8 - 106663.0*y10 + 486.14*y12 + 3432.0*y14;
		double o8 = 483737.0 - 498334.0*y2 - 217801.0*y4 - 161461.0*y6 - 55582.0*y8 + 1093.82*y10 + 3003.0*y12;
		double p8 = 70946.1 - 55600.0*y2 - 48153.3*y4 - 16493.7*y6 + 1012.79*y8 + 2002.0*y10;
		double q8 = 9504.65 - 3058.26*y2 - 1500.17*y4 + 567.164*y6 + 1001.0*y8;
		double r8 = 955.194 + 533.254*y2 + 198.876*y4 + 364.0*y6;
		double s8 = 126.532 + 40.5117*y2 + 91.0*y4;
		double t8 = 3.68288 + 14.0*y2;

		double top=a7+b7*x2+c7*x4+d7*x6+e7*x8+f7*x10+g7*x12+h7*x14+o7*x16+p7*x18+q7*x20+r7*x22+s7*x24+t7*x26;
		double bottom=a8+b8*x2+c8*x4+d8*x6+e8*x8+f8*x10+g8*x12+h8*x14+o8*x16+p8*x18+q8*x20+r8*x22+s8*x24+t8*x26+x28;
		return exp(y2-x2)*cos(2.0*x*y)-top/bottom;
	}
	return 0.0;
}*/

double KuntzHumlicekVoigtProfile(double x, double y)
{
//	Voigt Profile Calculation from paper:
//	M. Kuntz:	"A New Implementation of the Humlicek Algorithm for the Calculation of the Voigt Profile Function"
	double test1=fabs(x)+y;
	double x2=x*x;
	double y2=y*y;
	if(test1>=15)
	{
		//region 1
		double a1=y*(0.2820948+0.5641896*y2);
		double b1=0.5641896*y;
		double a2=0.25+y2*(1.0+y2);//0.25+y2+y4;
		double b2=-1.0+2*y2;
		return (a1+b1*x2)/(a2+x2*(b2+x2));
	}
	else if(test1>=5.5)
	{
		//region 2
		double a3=y*(y2*(y2*(y2*.56419f+3.10304f)+4.65456f)+1.05786f);
		double b3=y*(y2*(y2*1.69257f+.56419f)+2.962f);
		double c3=y*(y2*1.69257f-2.53885f);
		double d3=y*.56419f;
		double a4=y2*(y2*(y2*(y2+6.f)+10.5f)+4.5f)+.5625f;
		double b4=y2*(y2*(y2*4.f+6.f)+9.f)-4.5f;
		double c4=y2*(y2*6.f-6.f)+10.5f;
		double d4=y2*4.f-6.f;

		return (a3+x2*(b3+x2*(c3+x2*d3)))/(a4+x2*(b4+x2*(c4+x2*(d4+x2))));
	}
	else if(y-0.195*fabs(x)>-0.176)
	{
		// region 3
		double a5=y*(y*(y*(y*(y*(y*(y*(y*(y*.564224f+7.55895f)+49.5213f)+204.501f)+581.746f)+1174.8f)+1678.33f)+1629.76f)+973.778f)+272.102f;
		double b5=y*(y*(y*(y*(y*(y*(y*2.25689f+22.6778f)+100.705f)+247.198f)+336.364f)+220.843f)-2.34403f)-60.5644f;
		double c5=y*(y*(y*(y*(y*3.38534f+22.6798f)+52.8454f)+42.5683f)+18.546f)+4.58029f;
		double d5=y*(y*(y*2.25689f+7.56186f)+1.66203f)-.128922f;
		double e5=y*.564224f+9.71457e-4f;
		double a6=y*(y*(y*(y*(y*(y*(y*(y*(y*(y+13.3988f)+88.2674f)+369.199f)+1074.41f)+2256.98f)+3447.63f)+3764.97f)+2802.87f)+1280.83f)+272.102f;
		double b6=y*(y*(y*(y*(y*(y*(y*(y*5.f+53.5952f)+266.299f)+793.427f)+1549.68f)+2037.31f)+1758.34f)+902.306f)+211.678f;
		double c6=y*(y*(y*(y*(y*(y*10.f+80.3928f)+269.292f)+479.258f)+497.302f)+308.186f)+78.866f;
		double d6=y*(y*(y*(y*10.f+53.5952f)+92.7568f)+55.0293f)+22.0353f;
		double e6=y*(y*5.f+13.3988f)+1.49645f;

		return (a5+x2*(b5+x2*(c5+x2*(d5+x2*e5))))/(a6+x2*(b6+x2*(c6+x2*(d6+x2*(e6+x2)))));
	}
	else
	{
		//region 4
		double a7=y*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(2.35944f-y2*.56419f)-72.9359f)+571.687f)-5860.68f)+40649.2f)-320772.f)+1684100.f)-9694630.f)+40816800.f)-1.53575e8f)+4.56662e8f)-9.86604e8f)+1.16028e9f);
		double b7=y*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(23.0312f-y2*7.33447f)-234.143f)-2269.19f)+45251.3f)-234417.f)+3599150.f)-7723590.f)+86482900.f)-2.91876e8f)+8.06985e8f)-9.85386e8f)-5.60505e8f);
		double c7=y*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(97.6203f-y2*44.0068f)+1097.77f)-25338.3f)+98079.1f)+576054.f)-2.3818e7f)+22930200.f)-2.04467e8f)+2.94262e8f)+2.47157e8f)-6.51523e8f);
		double d7=y*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(228.563f-y2*161.358f)+8381.97f)-66431.2f)-303569.f)+2240400.f)+38311200.f)-41501300.f)-99622400.f)+2.70167e8f)-2.63894e8f);
		double e7=y*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(296.38f-y2*403.396f)+23507.6f)-66212.1f)-1.003e6f)+468142.f)+24620100.f)+5569650.f)+1.40677e8f)-63177100.f);
		double f7=y*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(125.591f-y2*726.113f)+37544.8f)+8820.94f)-934717.f)-1931140.f)-33289600.f)+4073820.f)-16984600.f);
		double g7=y*(y2*(y2*(y2*(y2*(y2*(y2*(-260.198f-y2*968.15f)+37371.9f)+79902.5f)-186682.f)-900010.f)+7528830.f)-1231650.f);
		double h7=y*(y2*(y2*(y2*(y2*(y2*(-571.645f-y2*968.15f)+23137.1f)+72520.9f)+153468.f)+86407.6f)-610622.f);
		double o7=y*(y2*(y2*(y2*(y2*(-575.164f-y2*726.113f)+8073.15f)+26538.5f)+49883.8f)-23586.5f);
		double p7=y*(y2*(y2*(y2*(-352.467f-y2*403.396f)+953.655f)+2198.86f)-8009.1f);
		double q7=y*(y2*(y2*(-134.792f-y2*161.358f)-271.202f)-622.056f);
		double r7=y*(y2*(-29.7896f-y2*44.0068f)-77.0535f);
		double s7=y*(-2.92264f-y2*7.33447f);
		double t7=y*-.56419f;
		double a8=y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2-3.68288f)+126.532f)-955.194f)+9504.65f)-70946.1f)+483737.f)-2857210.f)+14464700.f)-61114800.f)+2.11107e8f)-5.79099e8f)+1.17022e9f)-1.5599e9f)+1.02827e9f;
		double b8=y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*14.f-40.5117f)+533.254f)+3058.26f)-55600.f)+498334.f)-2849540.f)+13946500.f)-70135800.f)+2.89676e8f)-7.53828e8f)+1.66421e9f)-2.28855e9f)+1.5599e9f;
		double c8=y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*91-198.876f)-1500.17f)+48153.3f)-217801.f)-1063520.f)+1.4841e7f)-46039600.f)+63349600.f)-6.60078e8f)+1.06002e9f)-1.66421e9f)+1.17022e9f;
		double d8=y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*364-567.164f)-16493.7f)+161461.f)+280428.f)-6890020.f)-6876560.f)+1.99846e8f)+54036700.f)+6.60078e8f)-7.53828e8f)+5.79099e8f;
		double e8=y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*1001-1012.79f)-55582.f)+240373.f)+1954700.f)-5257220.f)-50101700.f)-1.99846e8f)+63349600.f)-2.89676e8f)+2.11107e8f;
		double f8=y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*2002-1093.82f)-106663.f)+123052.f)+3043160.f)+5257220.f)-6876560.f)+46039600.f)-70135800.f)+61114800.f;
		double g8=y2*(y2*(y2*(y2*(y2*(y2*(y2*(y2*3003-486.14f)-131337.f)-123052.f)+1954700.f)+6890020.f)+1.4841e7f)-13946500.f)+14464700.f;
		double h8=y2*(y2*(y2*(y2*(y2*(y2*(y2*3432+486.14f)-106663.f)-240373.f)+280428.f)+1063520.f)-2849540.f)+2857210.f;
		double o8=y2*(y2*(y2*(y2*(y2*(y2*3003+1093.82f)-55582.f)-161461.f)-217801.f)-498334.f)+483737.f;
		double p8=y2*(y2*(y2*(y2*(y2*2002+1012.79f)-16493.7f)-48153.3f)-55600.f)+70946.1f;
		double q8=y2*(y2*(y2*(y2*1001.f+567.164f)-1500.17f)-3058.26f)+9504.65f;
		double r8=y2*(y2*(y2*364+198.876f)+533.254f)+955.194f;
		double s8=y2*(y2*91.f+40.5117f)+126.532f;
		double t8=y2*14.f+3.68288f;

		return exp(y2-x2)*cos(2.0*x*y)-(a7+x2*(b7+x2*(c7+x2*(d7+x2*(e7+x2*(f7+x2*(g7+x2*(h7+x2*(o7+x2*(p7+x2*(q7+x2*(r7+x2*(s7+x2*t7)))))))))))))/(a8+x2*(b8+x2*(c8+x2*(d8+x2*(e8+x2*(f8+x2*(g8+x2*(h8+x2*(o8+x2*(p8+x2*(q8+x2*(r8+x2*(s8+x2*(t8+x2))))))))))))));
	}
	return 0.0;
}
