/**************************************************************************************************************
*
*       This file,
*
*                            /MacroCell.cpp,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#include "MacroCell.h"
#include "Grid.h"

#include "stringio.h"
#include "error.h"

MacroCell::MacroCell()
{
}

MacroCell::~MacroCell()
{
}

bool MacroCell::assignDefinition(char *definition)
{
	int grid_dimension=pGrid->getGridDimension();
	int k, i1, i2, g[3]=
	{
		pGrid->mapCellRangeIndexToGrid1DIndex(0),
		pGrid->mapCellRangeIndexToGrid1DIndex(1),
		pGrid->mapCellRangeIndexToGrid1DIndex(2)
	};
	class Grid1D *grid1D=pGrid->getExternalGrid1DPtr();
	char range_buffer[20];
	double c1, c2;
	// format: x1~x2:y1~y2:z1~z2
	if(numberOf(':', definition)!=grid_dimension-1)
	{
		errorIn(__FILE__, __LINE__, __func__);
		fprintf(stderr, "v-slab (\"%s\"): wrong number of colons (grid dimension)!\n", definition);
		return false;
	}
	for(k=0;k<grid_dimension;k++)
	{
		extractWordFromList(definition, k, ':', range_buffer);
		// format: c1~c2
		if(range_buffer[0]=='*' && range_buffer[1]=='\0')
		{
			c1=grid1D[g[k]].getA();
			c2=grid1D[g[k]].getB();
		}
		else if(sscanf(range_buffer, "%le~%le", &c1, &c2)!=2)
		{
			errorIn(__FILE__, __LINE__, __func__);
			fprintf(stderr, "error in v-slab definition(\"%s\"):\n", definition);
			fprintf(stderr, "range no. %d (\"%s\") has wrong format (correct: c1~c2)\n", k+1, range_buffer); 
			return false;
		}
		if(!grid1D[g[k]].findIndex(c1, &i1)\
			|| !grid1D[g[k]].findIndex(c2, &i2)\
			|| i2<=i1)
		{
			errorIn(__FILE__, __LINE__, __func__);
			fprintf(stderr, "error in v-slab definition (\"%s\"):\n", definition);
			fprintf(stderr, "range no. %d (\"%s\") is not representable by the %s grid\n", k+1, range_buffer, grid1D[g[k]].getName()); 
			return false;
		}

		xe_A[k]=c1;
		xe_B[k]=c2;
		
		xi_A[k]=pGrid->getInternalGrid1DPtr()[k].getGridPtr()[i1];
		xi_B[k]=pGrid->getInternalGrid1DPtr()[k].getGridPtr()[i2];
		
		iA[k]=i1;
		iB[k]=i2;
	
	}
	return true;
}

