/**************************************************************************************************************
*
*       This file,
*
*                            /Math/InterpolatedFunctionTemplate.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _InterpolatedFunctionTemplate_h
#define _InterpolatedFunctionTemplate_h

#include <stdio.h>
#include <stdlib.h>

#include "../stringio.h"
#include "../File.h"

#include "convert.h"

#define InterpolatedFunctionTemplate_MAX_CHARACTERS_PER_LINE		1011

template <class T> class InterpolatedFunctionTemplate
{
	char data_filename[2048];

protected:
	double *xp;
	T *yp;
	int no_points;

public:
	InterpolatedFunctionTemplate();
	virtual ~InterpolatedFunctionTemplate();
	bool readFromFile(char *filename, int no_rows=-1);
	bool writeToFile(char *filename);

	bool readFromFile(FILE *fptr, int no_rows=-1);
	bool writeToFile(FILE *fptr);
	bool allocate(int n);

	virtual T evaluate(double x)=0;

	// inline functtions
	inline char *getDataFilename(){return data_filename;}
	inline void definePoint(int index, double x, T y){xp[index]=x;yp[index]=y;}
	inline void defineX(int index, double x){xp[index]=x;}
	inline void defineY(int index, T y){yp[index]=y;}

	inline double xmin(){return xp[0];};
	inline double xmax(){return xp[no_points-1];};
	inline bool containsX(double x){return (x>=xp[0] && x<=xp[no_points-1]);}
	inline int getNOPoints(){return no_points;}
	inline double *getXPtr(){return xp;}
	inline T *getYPtr(){return yp;}
	inline bool isValid(){return yp && xp;}
};

#endif //_InterpolatedFunctionTemplate_h

