/**************************************************************************************************************
*
*       This file,
*
*                            /Math/LegendrePolynomial.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _LegendrePolynomial_h
#define _LegendrePolynomial_h

#include <stdio.h>

#include "TMatrix.h"
#include "Matrix.h"

class LegendrePolynomial
{
	// factor matrix
	enum Factor_e
	{
		F_l,
		F_lp1,
		F_lp2,
		F_twolp1
	};

	class TMatrix<int> factor;
	void calculateConstantFactors();

	double old_x;

protected:

	enum DataColumn_e
	{
		Basis,
		Primitive
	};
	

	// data holding basis polynomials and their primitives
	class TMatrix<double> LP;

	// Coefficients
	class Matrix Coefficient;
	int no_coefficients;
	
	void calculateBasisFunctions(const double& x);
	void calculateBasisPrimitives(const double& x);

	double calculateNorm();

	void calculateBasisFunctionsAndPrimitives(const double& x);
	bool normalize();

public:

	inline int degree(){return no_coefficients;}
	bool allocate(int n);

	inline double *getCoefficientPtr(){return Coefficient.getCoefficientPtr();}
	inline int getNOCoefficients(){return no_coefficients;}

	bool load(char *file_name);
	inline bool save(char *file_name){return Coefficient.save(file_name);}

	inline double evaluate(const double& x){return evaluate(x, 0);}
	double evaluate(const double& x, const int& index);
	double primitive(const double& x); 	// additive summand = 0

};

#endif //_LegendrePolynomial_h

