/**************************************************************************************************************
*
*       This file,
*
*                            /Math/LinearelyInterpolatedFunctionTemplate.cpp,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#include "LinearelyInterpolatedFunctionTemplate.h"

template<class T> T LinearelyInterpolatedFunctionTemplate<T>::evaluate(double x)
{
	int i=index;
#ifdef ZERO_BOUNDARIES
	if(x>=this->xp[this->no_points-1] || x<=this->xp[0])
		return T(0.0);
#else
	if(x>=this->xp[no_points-1])
		return this->yp[no_points-1];
	if(x<=this->xp[0])
		return this->yp[0];
#endif
	/*if(this->xp[i]<x)
	{
		while(this->xp[i]<x)
			i++;
	}
	else
	{
		while(this->xp[i]>=x)
			i--;
		i++;
	}*/
	int l, h;
	double xpi=this->xp[i];
	
	if(x==xpi)
		return this->yp[i];
	else if(x>xpi)
	{
		l=i;
		h=this->no_points-1;
	}
	else
	{
		l=0;
		h=i;
	}
	do
	{
		i=(l+h)/2;
		xpi=this->xp[i];
		if(x==xpi)
			return this->yp[i];
		else if(x>xpi)
			l=i;
		else
			h=i;
	}while(l+1!=h);
	index=i=h;
	
	//i is upper index now
	register double DeltaX=this->xp[i]-this->xp[i-1];
	T DeltaY=this->yp[i]-this->yp[i-1];
	register double fx=x-this->xp[i-1];
	T y=this->yp[i-1];
	DeltaY*=fx;
	DeltaY/=DeltaX;
	y+=DeltaY;
	return y;
}

template<class T> void LinearelyInterpolatedFunctionTemplate<T>::writeTemplate(FILE *fptr)
{
	fprintf(fptr,"height_1<TAB>value_11<TAB>...<TAB>value_1n<RET>\n");
	fprintf(fptr,"height_2<TAB>value_21<TAB>...<TAB>value_2n<RET>\n");
	fprintf(fptr,"...\n");
	fprintf(fptr,"height_m<TAB>value_m1<TAB>...<TAB>value_mn<RET>\n");
	fprintf(fptr,"EOF (end of file)\n");
}



