/**************************************************************************************************************
*
*       This file,
*
*                            /Math/Matrix3D.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _Matrix3D_h
#define _Matrix3D_h

#include "Vector3D.h"

class Matrix3D
{

	double a[3][3];

	friend class Vector3D;

	void internalCalculateInverse(class Matrix3D& im, const double& d) const;

public:
	enum InitType_e{ZERO, UNITY, VAL, ROTX, ROTY, ROTZ};

	Matrix3D();
	Matrix3D(enum InitType_e type, const double &val=0.0);
	Matrix3D(const class Vector3D& EulerAngles);
	Matrix3D(const class Vector3D& a0, const class Vector3D& a1, const class Vector3D& a2);	
	Matrix3D(class Vector3D *base_vector);

	~Matrix3D(){}
	
	// functions:
	void initializeWithSphericalBasisInColumns(const class Vector3D& p);
	double determinante() const;
	class Matrix3D inverse() const;
	class Matrix3D transpose() const;
	bool calculateInverse(class Matrix3D& im) const;

	inline class Vector3D column(const int& i){return Vector3D(a[0][i], a[1][i], a[2][i]);}
	inline class Vector3D row(const int& i){return Vector3D(a[i][0], a[i][1], a[i][2]);}

	// operators:
	inline double *operator[](const int& i){return a[i];}
	
	class Matrix3D operator*(const class Matrix3D& m) const;
	class Matrix3D& operator*=(const class Matrix3D& m);

	// mixed operators:
	class Vector3D operator*(const class Vector3D&);

	// friends:
	friend class Matrix3D axisRotation(const class Vector3D&, const double&);
	// friend class Matrix3D inverse(const class Matrix3D&);

};

class Matrix3D axisRotation(const class Vector3D&, const double&);
// inline class Matrix3D inverse(const class Matrix3D& m){return m.inverse();}

#endif // _Matrix3D_h


