/**************************************************************************************************************
*
*       This file,
*
*                            /Math/Octree.cpp,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#include "Octree.h"
#include "../error.h"

Octree::Octree()
{
	OXXX=0;
}

Octree::~Octree()
{
	if(OXXX)
		delete [] OXXX;
}

bool Octree::branch(class Vector3D& M)
{
	if(OXXX)
		delete [] OXXX;
	OXXX=new class Octree[8];
	if(!OXXX)
	{
		errorIn(__FILE__, __LINE__, __func__);
		fprintf(stderr, "error allocating octree!\n");
		return false;
	}
/*
	assign pointers:

	0	000	AAA
	1	001	AAB
	2	010	ABA
	3	011	ABB
	4	100	BAA
	5	101	BAB
	6	110	BBA
	7	111	BBB

	... and define boundaries of children:

                 * ------- * ------ *
                /|        /|       /|
               / |       / |      / |
              * ------- * ------ *  |
             /|  * ----/|- * ---/|- *
            / | /|    / |  |   / | /|
           * ------- * ------ *  |/ |
           |  * -----|- * ----|- *  |
           | /|  * --|--|- * -|-/|--*
           |/ | /    |  | /   |/ | /
           * ------- * ------ *  |/
           |  *------|- *-----|- *
           | /       | /      | /
           |/        |/       |/
           * ------- * ------ *

*/
	OAAA=OXXX+0;
	OAAA->A[2]=A[2];
	OAAA->A[1]=A[1];
	OAAA->A[0]=A[0];
	OAAA->B[2]=M[2];
	OAAA->B[1]=M[1];
	OAAA->B[0]=M[0];

	OAAB=OXXX+1;
	OAAB->A[2]=A[2];
	OAAB->A[1]=A[1];
	OAAB->A[0]=M[0];
	OAAB->B[2]=M[2];
	OAAB->B[1]=M[1];
	OAAB->B[0]=B[0];

	OABA=OXXX+2;
	OABA->A[2]=A[2];
	OABA->A[1]=M[1];
	OABA->A[0]=A[0];
	OABA->B[2]=M[2];
	OABA->B[1]=B[1];
	OABA->B[0]=M[0];

	OABB=OXXX+3;
	OABB->A[2]=A[2];
	OABB->A[1]=M[1];
	OABB->A[0]=M[0];
	OABB->B[2]=M[2];
	OABB->B[1]=B[1];
	OABB->B[0]=B[0];

	OBAA=OXXX+4;
	OBAA->A[2]=M[2];
	OBAA->A[1]=A[1];
	OBAA->A[0]=A[0];
	OBAA->B[2]=B[2];
	OBAA->B[1]=M[1];
	OBAA->B[0]=M[0];

	OBAB=OXXX+5;
	OBAB->A[2]=M[2];
	OBAB->A[1]=A[1];
	OBAB->A[0]=M[0];
	OBAB->B[2]=B[2];
	OBAB->B[1]=M[1];
	OBAB->B[0]=B[0];

	OBBA=OXXX+6;
	OBBA->A[2]=M[2];
	OBBA->A[1]=M[1];
	OBBA->A[0]=A[0];
	OBBA->B[2]=B[2];
	OBBA->B[1]=B[1];
	OBBA->B[0]=M[0];

	OBBB=OXXX+7;
	OBBB->A[2]=M[2];
	OBBB->A[1]=M[1];
	OBBB->A[0]=M[0];
	OBBB->B[2]=B[2];
	OBBB->B[1]=B[1];
	OBBB->B[0]=B[0];

	return true;
}



