/**************************************************************************************************************
*
*       This file,
*
*                            /Math/SphericalHarmonicsExpansion.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _SphericalHarmonicsExpansion_h
#define _SphericalHarmonicsExpansion_h

#include <memory.h>
#include <complex>

#include "AssociatedLegendrePolynomial.h"

using namespace std;

class SphericalHarmonicsExpansion: public AssociatedLegendrePolynomial
{
	double **alp_norm;
	double *alp_norm_data;	// norm of the associated Legendre polynomials

	complex<double> **c;
	complex<double> *coefficient_data, *azimuth_component;

	/*
		x 	xxx	xxxxx	xxxxxxx	xxxxxxxxx	xxxxxxxxxxx
		c0	c1	c2	c3	c4		c5
	*/

	void calculateAzimuthComponents(const double& phi);

public:

	SphericalHarmonicsExpansion();
	~SphericalHarmonicsExpansion();

	bool setOrder(int order);
	inline complex<double>& coefficient(const int& l, const int& m){return (l<=getOrder() && m<=l && m>=-l)?c[l][l+m]:(*((complex<double> *)0));}

	complex<double> evaluate(const double& theta, const double& phi);
	void resetCoefficients();

};

#endif // _SphericalHarmonicsExpansion_h


