/**************************************************************************************************************
*
*       This file,
*
*                            /Math/SplineFit.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _SplineFit_h
#define _SplineFit_h

#include "Spline.h"
#include "LeastSquaresMinimizer.h"

#include "Matrix.h"

class SplineFit: public Spline, public LeastSquaresMinimizer
{
	class Matrix data;		// contains two columns: x_m and y_m
	class Matrix lambda;		// smoothness constraints

	double lambda_a, lambda_b, constraint_strength;

	virtual bool defineX(class Vector& x);
	void setX(const class Vector& x);

	virtual bool calculateChiSquare(const class Vector& x, double& chiSquare);
	virtual bool calculateJAndH(const class Vector& x, double& chiSquare, class Vector& J, class Matrix& H);
	virtual bool calculateJ(const class Vector& x, double& chiSquare, class Vector& J){return false;}

protected:

	virtual void foundNewState(const class Vector& x, double new_chi_square, double old_chi_square);

public:

	SplineFit();

	inline void setConstraintStrength(double Sc){constraint_strength=Sc;}
	
	inline class Matrix& getData(){return data;}

	// bool initializeGrid(int n);
	bool loadDataPoints(char *file_name, int g);

};

#endif // _SplineFit_h


