/**************************************************************************************************************
*
*       This file,
*
*                            /Math/convert.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _convert_h
#define _convert_h

// #include "VectorND.h"
#include "../error.h"

#include <complex>

using namespace std;

template <class T> inline bool convert(char *text, T *pT)
{
	errorIn(__FILE__, __LINE__);
	fprintf(stderr," convert function not implemented!\n");
	return false;
}

template <class T> inline bool convert(T *pT, char *text)
{
	errorIn(__FILE__, __LINE__);
	fprintf(stderr, "convert function not implemented!\n");
	return false;
}

template <> inline bool convert(char *text, double *pDouble){*pDouble=atof(text);return true;}
template <> inline bool convert(double *pDouble, char *text){sprintf(text,"%le", *pDouble);return true;}

/*template <> inline bool convert(char *text, class VectorND *pVectorND)
{
	int i,d=numberOf('\t', text);
	try
	{
		*pVectorND=VectorND(d);
	}
	catch(class VectorNDMemoryException& vme)
	{
		vme.print(stderr);
		return false;
	}
	char buffer[20];
	double *px=pVectorND->getCoefficientBase();
	for(i=0;i<d;i++)
	{
		extractWordFromList(text,i,'\t',buffer);
		*px++=atof(buffer);
	}
	return true;
}

template <> inline bool convert(class VectorND *pVectorND, char *text)
{
	char buffer[20];
	double *px=pVectorND->getCoefficientBase();
	int i, d=pVectorND->getDimension();
	for(i=0;i<d-1;i++)
	{
		sprintf(buffer,"%lg\t", *px++);
		strcat(text, buffer);
	}
	sprintf(buffer,"%lg", *px++);
	strcat(text, buffer);
	return true;
}*/

template <> inline bool convert(char *text, complex<double> *pC)
{
	if(numberOf('\t', text)!=1)
		return false;
	char buffer[40];
	extractWordFromList(text,0,'\t',buffer);
	double real_part=atof(buffer);
	extractWordFromList(text,1,'\t',buffer);
	double imaginary_part=atof(buffer);
	*pC=complex<double>(real_part, imaginary_part);
	return true;
}

template <> inline bool convert(complex<double> *pC, char *text)
{
	sprintf(text,"%le\t%le", pC->real(), pC->imag());
	return true;
}

#endif //_convert_h
