/**************************************************************************************************************
*
*       This file,
*
*                            /Math/main_sphe.cpp,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "SphericalHarmonicsExpansion.h"

int main(int argc, char *argv[])
{
	SphericalHarmonicsExpansion SHE;
	/*
		she l m Ntheta Nphi
	*/
	if(argc!=5)
	{
		system("cat main_sphe.cpp");
		return 0;
	}
	int l=atoi(argv[1]);
	int m=atoi(argv[2]);
	int NTheta=atoi(argv[3]);
	int NPhi=atoi(argv[4]);

	if(!SHE.setOrder(l))
	{
		fprintf(stderr, "error setting order of associated Legendre polynomial!\n");
		return -1;
	}
	SHE.resetCoefficients();
	SHE.coefficient(l, m)=complex<double>(1.0, 0.0);
	int i, j;
	double theta, phi;
	for(i=0;i<=NTheta;i++)
	{
		theta=(M_PI*double(i))/(double(NTheta));
		for(j=0;j<=NPhi;j++)
		{
			phi=(2.0*M_PI*double(j))/(double(NPhi));
			printf(j==NPhi?"%le\n":"%le\t", SHE.evaluate(theta, phi).imag());
		}
	}
	return 0;
}










