/**************************************************************************************************************
*
*       This file,
*
*                            /Math/main_spline_fit.cpp,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#include <stdlib.h>

#include "SplineFit.h"
#include "../error.h"

int main(int argc, char *argv[])
{
	class SplineFit SFit;
	if(argc!=5)
	{
		fprintf(stderr, "usage: spline_fit data.txt g Sc mRc\n");
		return 0;
	}
	SFit.setConstraintStrength(atof(argv[3]));
	if(!SFit.loadDataPoints(argv[1], atoi(argv[2])))
	{
		errorIn(__FILE__, __LINE__, __func__);
		return -1;
	}
	/*if(!SFit.initializeGrid())
	{
		errorIn(__FILE__, __LINE__, __func__);
		fprintf(stderr, "error initializing grids\n");
		return 0;	
	}*/
	/*if(!SFit.initializeGrid(SFit.getData().getM()/7))
	{
		errorIn(__FILE__, __LINE__, __func__);
		return -1;
	}*/
	if(!SFit.fit(LeastSquaresMinimizer::LevenbergMarquardt, atof(argv[4])))
	{
		errorIn(__FILE__, __LINE__, __func__);
		fprintf(stderr, "error running LM spline fit!\n");
		return -1;
	}
	char file_name[1000];
	sprintf(file_name, "%s.fit", argv[1]);
	FILE *fptr=fopen(file_name, "w");
	if(!fptr)
	{
		errorIn(__FILE__, __LINE__, __func__);
		fprintf(stderr, "could not write result data matrix to file \"%s\"!\n", file_name);
	}
	else
	{
		SFit.getData().dumpTo(fptr);
		fclose(fptr);
	}
	SFit.save(stdout);
	return 0;
}



