/**************************************************************************************************************
*
*       This file,
*
*                            /McArtim_templates.cpp,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#include "Math/InterpolatedFunctionTemplate.cpp"
#include "Math/LinearelyInterpolatedFunctionTemplate.cpp"
#include "Math/TMatrix.cpp"
#include "Math/TVector.cpp"

#include <complex>
#include "Math/Vector3D.h"
#include "GeometryList.h"
#include "TrajectoryInformationObject.h"

using namespace std;

template InterpolatedFunctionTemplate<double>::InterpolatedFunctionTemplate();
template InterpolatedFunctionTemplate<double>::~InterpolatedFunctionTemplate();
template bool InterpolatedFunctionTemplate<double>::readFromFile(char*, int);
template double LinearelyInterpolatedFunctionTemplate<double>::evaluate(double);

template InterpolatedFunctionTemplate<complex<double> >::InterpolatedFunctionTemplate();
template InterpolatedFunctionTemplate<complex<double> >::~InterpolatedFunctionTemplate();
template bool InterpolatedFunctionTemplate<complex<double> >::allocate(int);
template bool InterpolatedFunctionTemplate<complex<double> >::readFromFile(char*, int);
template complex<double> LinearelyInterpolatedFunctionTemplate<complex<double> >::evaluate(double);

template TMatrix<complex <double> >::TMatrix();
template bool TMatrix<complex <double> >::allocate(int, int);
template TMatrix<complex <double> >::~TMatrix();

template TMatrix<Vector3D>::TMatrix();
template bool TMatrix<Vector3D>::allocate(int, int);
template TMatrix<Vector3D>::~TMatrix();

template TMatrix<double>::TMatrix();
template bool TMatrix<double>::allocate(int, int);
template TMatrix<double>::~TMatrix();
template void TMatrix<double>::dumpTo(FILE *, char const*, bool);

template TMatrix<int>::TMatrix();
template bool TMatrix<int>::allocate(int, int);
template bool TMatrix<int>::load(const char *file_name);
template TMatrix<int>::~TMatrix();

template TVector<complex <double> >::TVector();
template bool TVector<complex <double> >::allocate(int);
template TVector<complex <double> >::~TVector();

template TVector<int>::TVector();
template bool TVector<int>::allocate(int);
template TVector<int>::~TVector();

template TVector<double>::TVector();
template bool TVector<double>::allocate(int);
template TVector<double>::~TVector();

template TVector<bool>::TVector();
template bool TVector<bool>::allocate(int);
template TVector<bool>::~TVector();

template TVector<long int>::TVector();
template bool TVector<long int>::allocate(int);
template TVector<long int>::~TVector();

template TVector<struct GeometryList::TimeInterval_s>::TVector();
template bool TVector<struct GeometryList::TimeInterval_s>::allocate(int);
template TVector<struct GeometryList::TimeInterval_s>::~TVector();

template<> const char *TMatrix<int>::getFormatString(){return "d";}
template<> const char *TMatrix<double>::getFormatString(){return "lg";}

template TVector<class Vector3D>::TVector();
template bool TVector<class Vector3D>::allocate(int);
template TVector<class Vector3D>::~TVector();

template TVector<struct TrajectoryInformationObject::UpdateStruct_s>::TVector();
template bool TVector<struct TrajectoryInformationObject::UpdateStruct_s>::allocate(int);
template TVector<struct TrajectoryInformationObject::UpdateStruct_s>::~TVector();

