/**************************************************************************************************************
*
*       This file,
*
*                            /MolecularRRSCrossSection.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _MolecularRRSCrossSection_h
#define _MolecularRRSCrossSection_h

#include "constants.h"

#include "Math/prand.h"
#include "Math/Matrix.h"
#include "Math/Vector.h"

#include "GasMixtureDCS.h"

#include <stdio.h>

#define const_h		6.6260693e-34
#define const_c		speed_of_light
#define const_kB	k_B
#define const_RRSC	2901.51997426045		// 256*pi^5/27

class MolecularRRSCrossSection
{
	friend class RingEvent;

	public:

		enum RRSTransition_e
		{
			Stokes,
			AntiStokes
		};

		enum RRSWavelengthTransition_e
		{
			out_Stokes,
			out_AntiStokes,
			in_Stokes,
			in_AntiStokes
		};
	
		/*struct Parameters_s
		{
			struct gamma_s
			{
				double A, B, C, D;

				inline double evaluate(const double& wavelength)
				{
					register double s=1.0/(wavelength*1e6);
					return A*(B+C/(D-s*s));
				}
			} gamma;

			Parameters_s(const double& A, const double& B, const double& C, const double& D, \
				const double& B0, const int& gJe, const int& gJo, const int& I);

		};*/

		// public functions
		bool allocate(class Molecule *pM);
		
		void setTemperature(const double& temperature);
		void calculateCrossSections(const double& wavelength);

		double sampleWavelength(const double& p);
		void print(const int& j, FILE *fptr);

		// inline functions
		static inline int getJMax(){return J_max;}
		inline double getSigmaIn(){return sigma_in;}
		inline double getSigmaOut(){return sigma_out;}

		// static functions
		static bool setJMax(const int& n);	// to be called only once !!

	protected:

/*
	// static
		static struct Parameters_s DAMP[2];
		struct Parameters_s *pDAMP;
*/
		class Molecule *pMolecule;

		// thermodynamic properties
		class Vector wT;
		
		// wavelength dependent properties
		double wavelength;

		class Matrix lambda;
		class Matrix sigma;
		class Matrix scattering_probability;
		class Matrix cumulative_scattering_probability;
		
		// cross sections
		double sigma_out, sigma_in;
	
	private:

		// static members
		static int J_max;
		static class Matrix PlaczekTellerCoefficients;
};

#endif // _MolecularRRSCrossSection_h


