/**************************************************************************************************************
*
*       This file,
*
*                            /PhaseFunctions/DeltaPhaseFunction.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _DeltaPhaseFunction_h
#define _DeltaPhaseFunction_h

#include "PhaseFunction.h"

class DeltaPhaseFunction: public PhaseFunction
{
protected:

	double epsilon, peak_value;

	virtual double sampleMu(const double& alpha, double *pISWeight=0){return 1.0-alpha*epsilon;}

public:

	DeltaPhaseFunction(){}
	virtual ~DeltaPhaseFunction(){}
	
	// virtual parameter interface
	virtual inline int getNOParameters(){return 1;}
	virtual inline bool defineParameter(int index, double value){epsilon=value; peak_value=2.0/value; return true;}
	virtual double getParameter(int index){return epsilon;}

	virtual inline double evaluate(const double& mu){return mu>=1.0-epsilon?peak_value:0.0;}

	inline virtual bool setWavelength(const double& lambda){wavelength=lambda; return true;}

};

#endif //_DeltaPhaseFunction_h

