/**************************************************************************************************************
*
*       This file,
*
*                            /PhaseFunctions/HGQPhaseFunction.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _HGQPhaseFunction_h
#define _HGQPhaseFunction_h

#include "HenyeyGreensteinPhaseFunction.h"

#include "../Math/TMatrix.h"
#include "../Math/TVector.h"

class HGQPhaseFunction: public HenyeyGreensteinPhaseFunction
{

	//
	//	HGQ = Henyey-Greenstein Quadrature
	//
	enum PMCEntry_e
	{
		PMC_mu,	
		PMC_P11,
		PMC_P12,
		PMC_P22,
		PMC_P33,
		PMC_P34,
		PMC_P44,
		PMC_P12_P11,
		PMC_P22_P11,
		PMC_P33_P11,
		PMC_P34_P11,
		PMC_P44_P11,
		PMC_delta_mu,
		PMC_CDF,
		PMC_mu_q,
		PMC_hits,
		PMC_dimension
	};
	// phase function cache
	class TMatrix<double> PhaseMatrixCache;
	int phase_matrix_cache_resolution;

	int calculateCacheIndex(const double& mu);
	virtual void calculateCacheLine(const int& i, double& P11, double& P12, double& P22, double& P33, double& P34, double& P44);
	
	class TVector<int> PMC_index;

	// double *calculateCacheEntry(const int& i);	// returns pointer to cache line
	bool interpolateMuFromAlpha(double *pmu, const int& i, const double& alpha);
	void writeCache();

	// bool allocateMieCoefficients(int n);

	virtual double sampleMu(const double& alpha, double *pISWeight=0);

protected:

	bool enable_phase_matrix_cache, enable_interpolate_phase_matrix;
	bool createPhaseMatrixCache();
	
public:

	HGQPhaseFunction();

	inline void enableCache(bool enable=true, const int& resolution=0){enable_phase_matrix_cache=enable; phase_matrix_cache_resolution=resolution;}
	inline void enableInterpolation(bool enable=true){enable_interpolate_phase_matrix=enable;}
	
	virtual double evaluate(const double& mu);

	// to be implemented
	virtual double evaluateExactly(const double& mu)=0;
	virtual void calculatePhaseMatrixElements(const double& mu, double& P11, double& P12, double& P22, double& P33, double& P34, double& P44)=0;

	virtual class StokesMatrix& initializePhaseMatrix(class StokesMatrix& PM, const double& mu);
	virtual class StokesMatrix& initializeWeightedPhaseMatrix(class StokesMatrix& PM, const double& mu);

};

#endif // _HGQPhaseFunction_h


