/**************************************************************************************************************
*
*       This file,
*
*                            /PhaseFunctions/HenyeyGreensteinPhaseFunction.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _HenyeyGreensteinPhaseFunction_h
#define _HenyeyGreensteinPhaseFunction_h

#include "PhaseFunction.h"

class HenyeyGreensteinPhaseFunction: public PhaseFunction
{

	double g;
	double gg, ggg, gggg;					// accelerators
	double one_plus_gg, one_minus_gg, two_g;		// accelerators

	double h, p;					// Braak, de Haan, van der Mee, Hovenier, Travis

protected:

	virtual double sampleMu(const double& alpha, double *pISWeight=0);

public:	

	HenyeyGreensteinPhaseFunction();
	~HenyeyGreensteinPhaseFunction(){}
	
	virtual double evaluate(const double& mu);
	double evaluateCDF(const double& mu);

	// polarization service functions
	virtual class StokesMatrix& initializePhaseMatrix(class StokesMatrix& PM, const double& mu);
	virtual class StokesMatrix& initializeWeightedPhaseMatrix(class StokesMatrix& PM, const double& mu);

	// virtual parameter interface
	virtual inline int getNOParameters(){return 3;}
	virtual bool defineParameter(int index, double value);
	virtual double getParameter(int index){return g;}

	virtual void evaluateJacobian(double mu);
	virtual void evaluateHessian(double mu);

	// virtual void evaluateDerivatives(double mu, double *parameter_derivative);
	// virtual void evaluateDerivatives(double mu, double *pPhF, double *parameter_derivative);

};

#endif //_HenyeyGreensteinPhaseFunction_h


