/**************************************************************************************************************
*
*       This file,
*
*                            /PhaseFunctions/MiePhaseFunction.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _MiePhaseFunction_h
#define _MiePhaseFunction_h

#include "../MieModule.h"
#include "HGQPhaseFunction.h"
#include "../Math/LinearelyInterpolatedFunctionTemplate.h"

class MiePhaseFunction: public HGQPhaseFunction
{
	class MieModule MM;

	class LinearelyInterpolatedFunctionTemplate< complex<double> > ComplexRefractiveIndex;
	complex<double> refractive_index;

	int no_Mie_coefficients;
	bool allocateMieCoefficients(int n);

	class Vector pi, tau, gamma_p, gamma_m, TwoNp1oNNp1;
	class TMatrix< complex<double> > S_pp, S_pm, S_mp, S_mm;

	double extinction_coefficient, ssa;

	// FILE *phf_fptr;

	// NLogNormal DSD variables
	struct MieModule::Mode_s *mode;
	int no_modes, quadrature_resolution;

	// detailed DSD variables
	enum MieModule::DSDModel_e DSDModel;
	double *bin_radius, *NumberInBin;
	int no_bins;

public:

	MiePhaseFunction();

	void setModePtr(struct MieModule::Mode_s *, int);

	// detailed DSD information
	inline void defineDSDModel(enum MieModule::DSDModel_e m){DSDModel=m;}
	inline void defineRadiusList(double *r, int n){bin_radius=r; no_bins=n;}
	inline void defineBinCounts(double *NPtr){NumberInBin=NPtr;}

	virtual bool setWavelength(const double& lambda);
	virtual double evaluateExactly(const double& mu);
	virtual void calculatePhaseMatrixElements(const double& mu, double& P11, double& P12, double& P22, double& P33, double& P34, double& P44);

	bool initializeComplexRefractiveIndex(class Matrix& RIMatrix, int cluster_index);
	// inline functions
	inline class LinearelyInterpolatedFunctionTemplate< complex<double> >& getComplexRefractiveIndex(){return ComplexRefractiveIndex;}

	inline void setQuadratureResolution(int n){quadrature_resolution=n;}
	inline double getExtinctionCoefficient(){return extinction_coefficient;}
	inline double getSingleScatteringAlbedo(){return ssa;}
	inline double getAsymmetryParameter(){return getParameter(0);}

};

#endif // _MiePhaseFunction_h


