/**************************************************************************************************************
*
*       This file,
*
*                            /PhaseFunctions/TablePhaseFunction.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _TablePhaseFunction_h
#define _TablePhaseFunction_h

#include "PhaseFunction.h"
#include "../Math/Matrix.h"
#include "../Math/Vector.h"

class TablePhaseFunction: public PhaseFunction
{
	
	bool has_equidistant_mu_grid;

	class Matrix data;
	class Vector P11BinArea, AccumulatedP11BinArea;
	class Vector NormalizedP11;
	
	
	int no_angles, no_intervals; // , *mu_bin;
	bool initialize();	// call after change of data

	int findMuIndex(const double& mu);

	protected:

		virtual double sampleMu(const double& alpha, double *pISWeight=0);

	public:

		TablePhaseFunction();
		~TablePhaseFunction();

		virtual double evaluate(const double& mu);

		virtual class StokesMatrix& initializePhaseMatrix(class StokesMatrix& PM, const double& mu);
		virtual class StokesMatrix& initializeWeightedPhaseMatrix(class StokesMatrix& PM, const double& mu);

		virtual bool requiresFileInput(){return true;}
		virtual bool loadData(char *file_name);

};

#endif // _TablePhaseFunction_h


