/**************************************************************************************************************
*
*       This file,
*
*                            /PlaneParallelGrid.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _PlaneParallelGrid_h
#define _PlaneParallelGrid_h

#include "Grid.h"

class PlaneParallelGrid: public Grid
{

protected:

	int Z_Res;

public:

	// virtual interface
	inline virtual bool calculateGroundLayerIndex(){return internalX[2].findIndex(ground_level, &ground_layer_index);}
	virtual void convertHeightToGridHeight(double &h){}
	virtual double getVerticalExtension(const struct CellIndex_s& CI);
	virtual void convertToVector3D(const double& c0, const double& c1, const double& c2, class Vector3D& p){p[0]=c0; p[1]=c1; p[2]=c2;}

	// virtual double getGroundLevel();
	virtual void calculateSurfaceProjection(class Vector3D& position, double *px, double *py){*px=position[0]; *py=position[1];}
	virtual void convertEquidistantAlbedoMapCoordinates(double *pXA, double *pYA, double *pXB, double *pYB){}
	virtual void convertAlbedoMapGrids(double *x, int xres, double *y, int yres){}

	virtual void sampleTOAPoint(class Vector3D *pPosition, class Vector3D *pDirection, class Vector3D& SunPosition){}
	
	// BoxDetector things

	inline virtual void calculateEastAxis(double& x1, double& x2, double& x3, class Vector3D& n){n[0]=0.0;n[1]=1.0;n[2]=0.0;}
	inline virtual void calculateSouthAxis(double& x1, double& x2, double& x3, class Vector3D& n){n[0]=1.0;n[1]=0.0;n[2]=0.0;}
	inline virtual void calculateAltitudeAxis(double& x1, double& x2, double& x3, class Vector3D& n){n[0]=0.0;n[1]=0.0;n[2]=1.0;}
};

#endif //_PlaneParallelGrid_h

