/**************************************************************************************************************
*
*       This file,
*
*                            /Profile.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _Profile_h
#define _Profile_h

#include "ClusterMap.h"
#include "Math/Matrix.h"
#include "Math/StochasticVariable.h"
#include "Math/StokesVector.h"

class Profile: public ClusterMap
{
	
public:

	struct Cluster_s
	{
		bool valid;

		int no_cells;
		int *cell_index;

		double *input_data;
		double *optical_data;

		double *auxiliary_data;
	};
	
	
private:

	static bool b_output_polarized_quantities;

	// double *data;
	const char *profile_name;

	int *cell_index_data;

	int Jacobian_offset;
	int result_struct_index;

	bool retrieve_profile;
	
protected:

	// a cluster is a set of grid cells associated with a single profile value
	
	struct Cluster_s *cluster;
	int cluster_dimension, optical_cluster_dimension, auxiliary_data_dimension;

	virtual void setClusterDimensions(int input_cluster_dimension);
	// defines cluster dimensions above according to the dimension of the input cluster dimension
		
	bool enable_functional_calculation;

	bool initializeClusterArray();

	// double *auxiliary_data_base;

	class Matrix InputData, OpticalData, AuxiliaryData;

	bool allocateClusterData();
	
public:

	Profile();
	~Profile();
	
	inline void enableRetrieveProfile(bool enable=true){retrieve_profile=enable;}
	inline bool retrieveProfile(){return retrieve_profile;} 
	
	/*virtual void correctJacobiansForOpticalFactors();
	void getOpticalFactors(double *ptr);*/

	inline void setProfileName(const char *pname){profile_name=pname;}
	inline const char *getProfileName(){return profile_name;}

	void invalidateAllClusters();
	void invalidateCluster(const int& i);

	//inline void setOpticalClusterDimension(int d){optical_cluster_dimension=d;}
	static inline void enableOutputOfPolarizedQuantities(bool enable=true){b_output_polarized_quantities=enable;}

	//inline int getAuxiliaryDataDimension(){return auxiliary_data_dimension;}
	inline void resetAuxiliaryData(){AuxiliaryData.setToZero();}

	inline void setJacobianOffset(int o){Jacobian_offset=o;}
	inline int getJacobianOffset(){return Jacobian_offset;}

	inline void setResultStructIndex(int i){result_struct_index=i;}
	inline int getResultStructIndex(){return result_struct_index;}

	// -------------------------------------------

/*	void resetStochasticVariables();
	void resetStochasticVariableEstimates();
	void registerStochasticVariableWeightedEstimates(const double& TrajectoryWeight);

	double getStochasticVariablesMaximumPrecision();
	double getStochasticVariablesMinimumPrecision();

	void writeStochasticVariables(FILE *fptr, const char *output_format_string);
	void finishStochasticVariables();

	// access to stochastic variables
	inline int getNOStochasticVariables(){return no_stochastic_variables;}
	class StochasticVariable<double> *getScalarJacobianPtr(){return ScalarJacobianData;}
	class StochasticVariable<class StokesVector> *getVectorJacobianPtr(){return VectorJacobianData;}*/

	// -------------------------------------------
	
	inline int getClusterDimension(){return cluster_dimension;}
	inline struct Cluster_s *getClusterPtr(){return cluster;}
	inline struct Cluster_s *getClusterPtr(const int& cell_index){int ci=map[cell_index]; return ci>=0?cluster+ci:0;}

	inline bool isFunctionalCalculationEnabled(){return enable_functional_calculation;}
	inline void enableFunctionalCalculation(bool enable=true){enable_functional_calculation=enable;}

	bool loadClusterData(char *filename);
	virtual void writeAuxiliaryFiles();
};

#endif //_Profile_h













