/**************************************************************************************************************
*
*       This file,
*
*                            /RaytracerIO.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _RaytracerIO_h
#define _RaytracerIO_h

#include "BasicIO.h"
#include "RadiativeTransferEvent.h"
#include "SimulationObject.h"

class RaytracerIO: public BasicIO, public SimulationObject
{

public:
	RaytracerIO();
	~RaytracerIO();


	inline bool saveTrajectories(){return b_save_trajectories;}
	inline void enableSaveTrajectories(bool enable=true){b_save_trajectories=enable;}

	inline int getMaximumNumberOfTraceEvents(){return maximum_no_trace_events;}

	inline char *getOutputDirectory(){return dir_output_directory;}

	void formatScatteredTrajectoriesFilename(char *trajectories_filename);
	void formatUnscatteredTrajectoriesFilename(char *trajectories_filename);
	void formatLogFilename(char *log_filename);

protected:

	// IO ################################################################################

	// maximum cross section method...
	bool b_use_maximum_cross_section_method;	// indicates whether to use the maximum crosssection method
	double mcs_method_trigger;			// defines the maximal vertical optical thickness for the mcs method
	// end maximum cross section method
	bool b_simulate_refraction;			// do refraction or not

	double refraction_step_size;			// step size for the simulation of refraction
	double refraction_fitness;			// maximal direction difference for the ray exit direction, sun direction
	int refraction_iterations_maximum;		// maximal number of iteration steps in the refraction fitting procedure

	// sample Sun disc?
	bool b_consider_direct_light;			// consider direct (unscattered) sunlight
	// bool b_trace_until_sun;				// traces photon from the last scattering event to sun.

	int no_quadrature_trajectories;			// number of simulated trajectories
	int maximum_no_quadrature_trajectories;		// number of simulated trajectories
	int no_quadrature_points_per_cell;		
	// bool b_forced_scattering_mode;			// indicates the usage of forced first scatter event
	bool b_use_twilight_forcing;			// indicates e.g. a high terminator
	// double twilight_distance;
	bool b_automatic_switch_selection;		// automatic setting of some flags....

	// Neumann series truncation
	bool b_Neumann_series_truncation;		// limit number of scatter orders?
	int no_scatter_orders;				// maximum number of scatter orders...
	bool b_use_external_seed;			// is external seed used?
	int external_seed;				// random number generator seed

	int maximum_no_trace_events;				// maximum allowed number of traces events
	char dir_output_directory[BasicIO_string_size];		// output directoy
	char output_filename_prefix[BasicIO_string_size];	// output filename prefix
	bool b_save_trajectories;				// ...
	bool b_ASCII_trajectory_output;

	// End of IO ##########################################################################

	// streams
	FILE *log_stream, *scattered_stream, *unscattered_stream;

	bool openStreams();
	void closeStreams();

	void writeSingleEventASCII(FILE *fptr, struct RadiativeTransferEvent_s *pRTEvent);
	void logEvent(struct RadiativeTransferEvent_s *pRTEvent);

};

#endif // _RaytracerIO_h


