/**************************************************************************************************************
*
*       This file,
*
*                            /RingEvent.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _RingEvent_h
#define _RingEvent_h

#include "Profile.h"
#include "MolecularRRSCrossSection.h"
#include "GasMixtureDCS.h"

#include "Math/LinearelyInterpolatedFunctionTemplate.h"

#include <stdio.h>

//#define	p_N2		( 0.78084 / 0.9903 )
//#define	p_O2		( 0.20946 / 0.9903 )
#define	p_N2		0.7809
#define	p_O2		0.2095

class Atmosphere;

class RingEvent
{
	class MolecularRRSCrossSection N2, O2;

	double sigma_in, sigma_out;	// total cross section
	double sigma_in_N2, sigma_in_O2;

	// importance sampling weight 
	double sigma_in_sigma_out_ratio;

	double lambda;			// center wavelength
	double temperature;

	struct Profile::Cluster_s *pAirCluster;

public:

	// functions operating on air 
	double calculateCrossSections(const double& wavelength);// returns the extinction coefficient for out-scattering
	double sampleWavelength();

	void getInScatteringWavelengths(double *pwavelength);
	void getInScatteringCrossSections(double *psigma);

	// void resetTauForced();
	void print(FILE *fptr, bool detailed=false);

	// inline functions
	bool allocate();
	void setTemperature(const double& temperature);

	inline void associateWithAirCluster(struct Profile::Cluster_s *pCluster){pAirCluster=pCluster;}
	inline void calculateKineticConstants(){setTemperature(pAirCluster->input_data[1]);}
	// inline double getTotalCrossSection(){return sigma;}
	
	inline double getRatioSigmaInSigmaOut(){return sigma_in_sigma_out_ratio;}

};

#endif //_RingEvent_h


