/**************************************************************************************************************
*
*       This file,
*
*                            /Screen.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _Screen_h
#define _Screen_h

#include "Simulation.h"
#include "Depiction/TraceEventCollection.h"
#include "Depiction/SurfaceGridPointCollection.h"
#include "Depiction/RenderScreen.h"
#include "Depiction/PointCollectionRenderList.h"

class Screen : public Simulation
{
	protected:

		// IO #########################################################
		int width, height;
		char fn_perspective[BasicIO_string_size];	// perspective file
		char event_mask[1000];
		char fn_outbase[BasicIO_string_size];
		char fn_simulation[BasicIO_string_size];
		// END OF IO ##################################################
		
		bool *pSimulationControlFlag;

		char trajectory_filename[2047];

		// point collections...
		class TraceEventCollection TraceEvents;
		class SurfaceGridPointCollection SurfaceGridPoints;
		class RenderScreen PointCloudScreen;

		bool enable_trace_event[1000];
		int maximal_event_type_index;

		// variables for mouse operations
		class PointCollectionRenderList PCRenderList;

		bool loadScatteredTrajectories();
		bool loadUnscatteredTrajectories();
		
		virtual void post_process_run(const int& run_id);

	public:

		Screen();
		~Screen(){}
		
		bool initialize(int argc, char *argv[]);

};

#endif //_Screen_h





