/**************************************************************************************************************
*
*       This file,
*
*                            /SimulationObject.cpp,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#include "SimulationObject.h"

#include "Simulation.h"
#include "RingEvent.h"

class Grid *SimulationObject::pGrid=0;
int SimulationObject::no_cells=0;

class Simulation *SimulationObject::pSimulation=0;	
class Raytracer *SimulationObject::pRaytracer=0;
class Surface *SimulationObject::pSurface=0;
class Geometry *SimulationObject::pGeometry=0;

class Atmosphere *SimulationObject::pAtmosphere=0;

class Gas *SimulationObject::gas=0;
int SimulationObject::no_gases=0;

class Particle *SimulationObject::particle=0;
int SimulationObject::no_particles=0;

void SimulationObject::setSimulationPtr(class Simulation *SimulationPtr)
{
	pSimulation=SimulationPtr;
	pRaytracer=pSimulation->getRaytracerPtr();
	pGeometry=pSimulation->getGeometryPtr();

	pSurface=pSimulation->getSurfacePtr(); 
	
	pAtmosphere=pSimulation->getAtmospherePtr();

	updateAtmosphere();
}

void SimulationObject::formatOutputDirectory(char *path)
{
	sprintf(path, "%s%s", pRaytracer->FunctionalLayer::getBaseDirectory(), pRaytracer->getOutputDirectory());
}

void SimulationObject::updateAtmosphere()
{
	// setup gas and particle access
	gas=pAtmosphere->getGasPtr();
	no_gases=pAtmosphere->getNOGases();

	particle=pAtmosphere->getParticlePtr();
	no_particles=pAtmosphere->getNOParticles();
}

void SimulationObject::updateGrid()
{
	pGrid = pSimulation->getGridPtr();
	no_cells = pGrid -> getNOCells();
}

