/**************************************************************************************************************
*
*       This file,
*
*                            /VarianceReductionLayer.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _VarianceReductionLayer_h
#define _VarianceReductionLayer_h

#include "VarianceReductionGrid.h"
#include "BasicIO.h"
#include "SimulationObject.h"

class VarianceReductionLayer: public BasicIO, public SimulationObject
{
	public:

		VarianceReductionLayer();
		~VarianceReductionLayer();

		bool setup(int maximum_no_trace_events);

	protected:
		// IO ----------------------------------------
		// variance reduction
		// bool b_adaptive_splitting;
		// bool b_enable_directional_importance_sampling;
		bool b_generate_variance_reduction_parameter_file;
		bool b_load_variance_reduction_parameter_file;

		int maximum_recursion_depth;	// limit memory usage for variance reduction methods
		double mu_split;
		double w_split, w_russian_roulette;
		double p_split;
		int no_mu_bins, no_phi_bins;
		bool b_importance_sampling_on_root_trajectory;
		char fn_variance_reduction_parameter_file[BasicIO_string_size];
		
		// END OF IO ---------------------------------------------------------------
	
		class VarianceReductionGrid VRGrid;

		// variance reduction

		//	VarianceReductionParameterMatrix:
		//	
		//	0:	sum of weight * contribution
		//	1:	sum of weights
		// 	2:	weighted contribution
		// 	3:	normalized weighted contribution
		//	4:	splitting limit
		//	5: 	russian roulette limit
		//	6:	psi sampling probability
		//	7:	g_IS

		// class Matrix VarianceReductionParameterMatrix;
		double psi_sampling_probability;
		double delta_mu;
		double g_sun;

		bool update_variance_reduction_parameters;
		// class Matrix ScatterOrderContributionMatrix;	// first column: scatter order contribution, second column cumulative relative contribution

		// void calculatePsiSamplingProbabilities(class Result *pResult);
		// void updateVarianceReductionParameters();
};

#endif // _VarianceReductionLayer_h


