/**************************************************************************************************************
*
*       This file,
*
*                            /constants.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _constants_h
#define _constants_h 

#include <math.h>

#define M_2PI			6.283185307179586476925286766558

#ifdef WIN32
	#if not defined MINGW
		#ifdef USE_QT
			#include <math.h>
		#else
			#ifndef M_PI
				#define M_PI		3.1415926535897932384626433832795028841971693993751
			#endif
			#ifndef M_E
				#define M_E		2.71828182845904523536028747135266249775724709367
			#endif
		#endif
	#endif
#endif

#define SQRT_2PI		2.50662827

#define M_4PI_inverse			0.0795774715459477
#define EARTH_RADIUS			6377.5001

#define AU				1.49597870691e8
#define SUNDIAMETER			1392000
// #define SUN_THETA			0.26656512
#define SUN_SOLID_ANGLE_RADIUS		(0.5*SUNDIAMETER/AU)
#define SUN_SOLID_ANGLE			(M_PI*SUN_SOLID_ANGLE_RADIUS*SUN_SOLID_ANGLE_RADIUS)

#define speed_of_light			299792458
#define k_B				1.380650424e-23
#define Avogadro			6.02214179e23
#define PlanckConstant			6.62606957e-34


#endif //_constants_h
