/**************************************************************************************************************
*
*       This file,
*
*                            /main_SSA.cpp,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#include "Simulation.h"
#include "version.h"

#include <locale.h>

int main(int argc, char *argv[])
{
	setlocale(LC_ALL, "C");
	if(argc==1)
	{
		printf("usage: AnalyticalSSA [simulation.conf [-t]|-h [-c directory|-l|IOName]]\n");
		printVersion(__FILE__, stdout);
		return 0;
	}
	Simulation simulation;
	if(!strcmp(argv[1], "-h"))
	{
		class Gas gas;
		class Particle particle;
		class GridIO gridIO;
		class BasicIO *IOFileList[]=
		{
			simulation.getAtmospherePtr(),
			simulation.getRaytracerPtr()->getFunctionalLayerIOPtr(),
			&gas,
			simulation.getGeometryPtr(),
			&gridIO,
			&particle,
			simulation.getRaytracerPtr()->getRaytracerIOPtr(),
			&simulation,
			simulation.getSurfacePtr()
		};
		int i, no_ptrs=sizeof(IOFileList)/sizeof(class BasicIO *);
		if(!argv[2] || !strcmp(argv[2], "-l"))
			for(i=0;i<no_ptrs;i++)
				printf("%s\n", IOFileList[i]->getIOName());
		else if(!strcmp(argv[2], "-c"))
			simulation.createConfigurationSkeleton(argv[3]);
		else
		{
			for(i=0;i<no_ptrs;i++)
				if(!strcmp(argv[2], IOFileList[i]->getIOName()))
				{
					printf("# %s.conf\n", argv[2]);
					IOFileList[i]->writeTemplate(stdout);
					break;
				}
			// check for fall through
			if(i==no_ptrs)
			{
				errorIn(__FILE__, __LINE__, __func__);
				fprintf(stderr, "no IO template for type %s!\n", argv[2]);
				return -1;
			}
		}
	}
	else
	{
		if(!simulation.readAndLoadFiles(argv[1]) || !simulation.initialize())
		{
			errorIn(__FILE__, __LINE__, __func__);
			fprintf(stderr, "failed to load simulation!\n");
			return -1;
		}
		if(argv[2] && !strcmp(argv[2], "-t"))
			return simulation.processTable(Simulation::AnalyticalSSA)?0:-1;
		if(!simulation.run(Simulation::AnalyticalSSA))
			return -1;
		return 0;
	}
	return 0;
}



