/**************************************************************************************************************
*
*       This file,
*
*                            /stringio.h,
*
*        is part of the publically available source code of McArtim 3.
*
*       You may not use the source code in ANY way for commercial purposes,
*       i.e. to generate profit from any derivations (modifications and/or compilations, etc.)
*       of the source code without my explicit permission. ANY commercial usage has to be allowed by me,
*       Tim Deutschmann, and will be charged with a profit share of at least 5%.
*
*       You may freely study, modify, compile and redistribute the following source code
*
*	                     FOR SCIENTIFIC PURPOSES ONLY.
*
*       The modified source code has to be made publically available at the latest on the day of
*       publishing the effects of the modification. All derivations of this source code shall remain
*       to be 'open source'.
*	
*       If you use the source code or the programs compiled from the original or
*       from the modified files you must properly cite the author of the original code by refering
*       to the following publications:
*	
*
*       Tim Deutschmann: Atmospheric Radiative Transfer Modelling with Monte Carlo Methods,
*       master thesis, University of Heidelberg, 2009.
*
*       Deutschmann, T., Beirle, S., Frieß, U., Grzegorski, M., Kern, C., Kritten, L.,
*       Platt, U., Prados-Román, C., Pukı̄te, J., Wagner, T., Werner, B., and Pfeilsticker, K.:
*       The Monte Carlo Atmospheric Radiative Transfer Model McArtim:
*       Introduction and Validation of Jacobians and 3D Features,
*       Journal of Quantitative Spectroscopy and Radiative Transfer, 112(6):1119-1137, 2011.
*
*       Tim Deutschmann: On Modeling Elastic and Inelastic Polarized Radiation Transport in the
*       Earth Atmosphere with Monte Carlo Methods, PhD thesis, University of Leipzig, 2014.
*
*       The official webpage of the McArtim source code 3.0 is
*
*                        http://www.tim-deutschmann.de/McArtim/index.html
*       
*       You may contact me through Tim.Deutschmann@posteo.de.
*
*       Have fun with the code!
*
*
*
*	                                                 Tim Deutschmann, Dossenheim, May 11th 2018.
*
**********************************************************************************************************/

#ifndef _stringio_h
#define _stringio_h

#include <string.h>
#include <time.h>
#include <list>
#include <string>

using namespace std;

int numberOf(char character, const char *text);
bool extractWordFromList(const char *text, int index, char separation_character, char *buffer);
bool extractPathFromFilename(const char *filename, char *path_buffer, char *name=0);
void remove(char character, char *text);
bool extractBracketExpression(const char *text, char b1, char b2, char *buffer);

bool extractBool(const char *value_string, bool *aboolptr);
inline const char *getBoolString(bool abool){return abool?"yes":"no";}

char *strncpyz(char *destination, const char *source, size_t n);

void condenseLine(char *line_buffer);	// removes space/tab/extra character polutions between the columns and replace them by one TAB character
bool calculateSecondsInEpoche(int *pSiE, char *date_string, char *time_string);
void formatTimeFromSecondsInEpoche(time_t SiE, char *time_string);

void createListOfStrings(char *x_separated_list, char x, class list<string>& list_of_strings, const char *empty_list_string="none");

#endif //_stringio_h

