#include "PathViewer.h"
#include "../../minmax.h"

PathViewer::PathViewer(int argc, char *argv[])
{
	if(load(argv[1]))
	{
		width=400;
		height=300;
		createSimpleWindow("McArtim PathViewer",0,0,width,height, 0, pY->YBlackPixel(), pY->YBlackPixel());
		left_mouse_button_down=false;
		middle_mouse_button_down=false;
		right_mouse_button_down=false;
		PointCloudScreen.setPerspective(900.0,1590.0);
		rotation_angle=-0.015;
		for(int i=0;i<1000;i++)
			enable_trace_event[i]=true;
		enable_trace_event[TLC_REFRACTIVE_INDEX]=false;
		/*enable_trace_event[TLC_TOTAL_ABSORPTION_COEFFICIENT]=true;
		enable_trace_event[TLC_GROUND]=true;
		enable_trace_event[TLC_EPSILON_SCATTERER]=true;
		enable_trace_event[TLC_EPSILON_RAYLEIGH]=true;
		enable_trace_event[TLC_EPSILON_ROTATIONAL_RAMAN]=true;
		enable_trace_event[TLC_EPSILON_AEROSOL]=true;*/
		resetMarkers();
		raytracer.b_save_trajectories=true;
		SurfaceGridPoints.setSurfaceGridPtr(&solidSurface);
		SurfaceGridPoints.setGranularity(0.1);
		SurfaceGridPoints.setGridColor(0xa0a0a0);
	}
}

void PathViewer::OnCreateWindow()
{
	if(!resizeScreen(width, height))
		keep_open=false;
	loadScatteredTrajectories();
	struct ScreenPoint_s c;
	c.x=width/2;
	c.y=height/2;
	int imin=findIndexOfClosestPoint(c);
	struct ColoredPoint_s *cp=TraceEvents.getColoredPointBase();
	Vector3D delta_p=-cp[imin].point;
	TranslatePoints(delta_p);
	RenderClientWindow();
	ResetPerspective();
}

bool PathViewer::resizeScreen(int xres, int yres)
{
	if(!PointCloudScreen.allocate(xres,yres,true))
	{
		fprintf(stderr,"could not resize to %dx%d!\n", xres, yres);
		keep_open=false;
	}
	else if(!DrawingBoard.setSize(width, height, 4))
	{
		fprintf(stderr,"could not resize drawing board to %dx%d!\n", xres, yres);
		keep_open=false;
	}
	/*else
		printf("resized to %dx%d\n", width, height);*/
	return keep_open;
}

bool PathViewer::OnConfigureNotify(XConfigureEvent *pXConfigureEvent)
//bool PathViewer::OnResize(XResizeRequestEvent *pXResizeRequestEvent)
{
	//int w=pXResizeRequestEvent->width;
	//int h=pXResizeRequestEvent->height;
	int w=pXConfigureEvent->width;
	int h=pXConfigureEvent->height;
	if(w != width || h !=height)
	{
		width = w;
		height = h;
		if(!resizeScreen(width, height))
			keep_open=false;
	}
	return true;
}

void PathViewer::RenderClientWindow()
{
	void *destptr=DrawingBoard.getDataPtr();
	void *srcptr=PointCloudScreen.getDataPtr();
	if(destptr && srcptr)
	{
		PointCloudScreen.clearScreen();
		PointCloudScreen.renderPointCollection(&TraceEvents);
		PointCloudScreen.renderPointCollection(&SurfaceGridPoints);
		//fputc('.',stdout);
		memcpy(destptr,srcptr, 4*width*height);
		DrawingBoard.putImage(this, 0, 0);
		XSync(pDisplay, true);
	}
}

void PathViewer::drawMarkers()
{
/*	const int marker_size=4,marker_edge=2;
	int i,x,y;
	struct Marker_s *marker[3]=
	{
		&MarkerLeft,
		&MarkerRight,
		&MarkerCenter
	};
	int no_markers=sizeof(marker)/sizeof(marker[0]);
	CPen *pOldPen,WhitePen(PS_SOLID,1,0x00ffffff); //screaming blue
	pOldPen=pDC->SelectObject(&WhitePen);

	//Vector3D *point3D=TraceEvents.GetPointBase();
	struct ColoredPoint_s *cp=TraceEvents.getColoredPointBase();
	char marker_text[30];
	pDC->SetBkMode(TRANSPARENT);
	COLORREF old_text_color=pDC->GetTextColor();
	pDC->SetTextColor(0xffff);
	bool found_unset_marker=false;
	for(i=0;i<no_markers;i++)
	{
		if(marker[i]->index!=-1)
		{
			//marker[i]->point=point3D[marker[i]->index];
			marker[i]->point=cp[marker[i]->index].point;
			PointCloudScreen.toScreenPoint(marker[i]->point,x,y);
			//marker[i]->screen_point=CPoint(x,y);
			pDC->MoveTo(x-marker_size+marker_edge,y-marker_size);
			pDC->LineTo(x-marker_size,y-marker_size);
			pDC->LineTo(x-marker_size,y+marker_size);
			pDC->LineTo(x-marker_size+marker_edge+1,y+marker_size);
			pDC->MoveTo(x+marker_size-marker_edge,y-marker_size);
			pDC->LineTo(x+marker_size,y-marker_size);
			pDC->LineTo(x+marker_size,y+marker_size);
			pDC->LineTo(x+marker_size-marker_edge-1,y+marker_size);
			sprintf(marker_text,"%d",i+1);
			pDC->TextOut(x+marker_size+1,y+marker_size+1,CString(marker_text));
		}
		else
			found_unset_marker=true;
	}
	if(MarkerLeft.index!=-1 && MarkerRight.index!=-1)
	{
		sprintf(marker_text,"d(|1->2|) : %.3lfkm",abs(MarkerLeft.point-MarkerRight.point));
		pDC->TextOut(3,3,CString(marker_text));
	}
	if(!found_unset_marker)
	{
		Vector3D dA,dB,delta;
		for(i=0;i<no_markers;i++)
		{
			delta=marker[(i+1)%3]->point-marker[i]->point;
			dA=norm(delta);
			delta=marker[(i+2)%3]->point-marker[i]->point;
			dB=norm(delta);
			sprintf(marker_text,"angle %d  : %.2lf°",i+1,Degree(acos(correctMu(dA*dB))));
			pDC->TextOut(3,20+i*19,CString(marker_text));
		}
	}
	pDC->SelectObject(pOldPen);
	pDC->SetTextColor(old_text_color);
*/
}

void PathViewer::resetMarkers()
{
	MarkerLeft.index=-1;
	MarkerCenter.index=-1;
	MarkerRight.index=-1;
}

bool PathViewer::OnExpose(XExposeEvent *pXExposeEvent)
{
	//printf("Expose\n");
	if(TraceEvents.isValid())
		RenderClientWindow();
	drawMarkers();
	return true;
}

bool PathViewer::OnMotion(XMotionEvent *pXMotionEvent)
{
	int x=pXMotionEvent->x, y=pXMotionEvent->y;		/* pointer x, y coordinates in event window */
	unsigned int state=pXMotionEvent->state;	/* key or button mask */
	//printf("motion to (%d|%d), state %u\n", x, y, state);
	double dx=rotation_angle*(x-mouse_point.x);
	double dy=rotation_angle*(y-mouse_point.y);
	if(state&STATE_MASK_LEFT_BTN)
	{		
		Matrix3D rotz=Matrix3D(Matrix3D::ROTZ,dx),roty=Matrix3D(Matrix3D::ROTY,-dy);
		if(x!=mouse_point.x)
			TransformPoints(rotz);
		if(y!=mouse_point.y)		
			TransformPoints(roty);
		RenderClientWindow();
	}
	else if(state&STATE_MASK_RIGHT_BTN)
	{		
		Matrix3D rotx=Matrix3D(Matrix3D::ROTX,-dx),roty=Matrix3D(Matrix3D::ROTY,dy);
		if(x!=mouse_point.x)
			TransformPoints(rotx);
		/*if(y!=mouse_point.y)
			TransformPoints(roty);*/
		RenderClientWindow();
	}
	mouse_point.x=x;
	mouse_point.y=y;
	return true;
}

bool PathViewer::loadTrajectories(char *filename, FILE *logstream)
{
	fprintf(logstream, "reloading events from \"%s\"...\t",filename);
	if(!TraceEvents.loadFromFile(filename))
	{
		fprintf(logstream, "failed!\n");
		return false;
	}
	fprintf(logstream, "success!\n");
	strcpy(trajectory_filename, filename);
	return true;
}

bool PathViewer::loadScatteredTrajectories(FILE *logstream)
{
	char filename[2047];
	raytracer.formatScatteredTrajectoriesFilename(filename);
	return loadTrajectories(filename, logstream);
}

bool PathViewer::loadUnscatteredTrajectories(FILE *logstream)
{
	char filename[2047];
	raytracer.formatUnscatteredTrajectoriesFilename(filename);
	return loadTrajectories(filename, logstream);
}

bool PathViewer::OnKeyPress(XKeyPressedEvent *pXKeyPressedEvent)
{
	unsigned int keycode=pXKeyPressedEvent->keycode, state=pXKeyPressedEvent->state;
	switch(keycode)
	{
		case 24://q
			keep_open=false;
			break;
		case 39://s
			PointCloudScreen.toggleStereoMode();
			RenderClientWindow();
			resetMarkers();
			break;
		case 27://r
		{
			if(!TraceEvents.reload(enable_trace_event) || !SurfaceGridPoints.initialize())
			{
				fprintf(stderr,"something went wrong during reload of RT events...\n");
				if(!loadTrajectories(trajectory_filename))
				{
					keep_open=false;
					return true;
				}
			}
			restorePerspective();
			/*PointCloudScreen.clearScreen();
			PointCloudScreen.renderPointCollection(&TraceEvents);
			PointCloudScreen.renderPointCollection(&SurfaceGridPoints);*/
			RenderClientWindow();
			resetMarkers();
			break;
		}
		default:
			printf("keycode %u, state %u\n", keycode, state);
	}
	return true;
}

bool PathViewer::OnButtonPress(XButtonPressedEvent *pXButtonPressedEvent)
{
	int x=pXButtonPressedEvent->x, y=pXButtonPressedEvent->y;		/* pointer x, y coordinates in event window */
	unsigned int state=pXButtonPressedEvent->state;	/* key or button mask */
	unsigned int button=pXButtonPressedEvent->button;	/* detail */
	//printf("button %u pressed, state %u, position=(%d|%d)\n", button, state, x, y);
	mouse_point.x=x;
	mouse_point.y=y;
	switch(button)
	{
		case 1:
			left_mouse_button_down=true;
			if(state & STATE_MASK_CTRL)
			{
				int imin=findIndexOfClosestPoint(mouse_point);
				struct ColoredPoint_s *cp=TraceEvents.getColoredPointBase();
				Vector3D delta_p=-cp[imin].point;
				TranslatePoints(delta_p);
				RenderClientWindow();
			}
			if(state & STATE_MASK_SHIFT)
			{
				MarkerLeft.index=findIndexOfClosestPoint(mouse_point);
				RenderClientWindow();
			}
			break;
		case 2:
			middle_mouse_button_down=true;
			break;
		case 3:
			right_mouse_button_down=true;
			if(state & STATE_MASK_SHIFT)
			{
				MarkerRight.index=findIndexOfClosestPoint(mouse_point);
				RenderClientWindow();
			}
			break;
		default:;
	}
	return true;
}

bool PathViewer::OnButtonRelease(XButtonReleasedEvent *pXButtonReleasedEvent)
{
	int x=pXButtonReleasedEvent->x, y=pXButtonReleasedEvent->y;		/* pointer x, y coordinates in event window */
	unsigned int state=pXButtonReleasedEvent->state;	/* key or button mask */
	unsigned int button=pXButtonReleasedEvent->button;	/* detail */
	//printf("button %u released, state %u, position=(%d|%d)\n", button, state, x, y);
	mouse_point.x=x;
	mouse_point.y=y;
	switch(button)
	{
		case 1:
			left_mouse_button_down=false;
			break;
		case 2:
			middle_mouse_button_down=false;
			if(state & STATE_MASK_SHIFT)
			{
				MarkerCenter.index=findIndexOfClosestPoint(mouse_point);
				RenderClientWindow();
			}
			break;
		case 3:
			right_mouse_button_down=false;
			break;
		case 4:
		case 5:
		{
			
			int zDelta;
			if(button==4)
				zDelta=120;
			else
				zDelta=-120;
			double distance=PointCloudScreen.getPerspectiveDistance();
			double factor=PointCloudScreen.getPerspectiveFactor();
			if(state&STATE_MASK_CTRL && !(state&STATE_MASK_SHIFT))
			{
				distance-=0.2109*(double)zDelta;
				PointCloudScreen.setPerspective(distance,factor);
				OnExpose(0);
			}
			else if(state&STATE_MASK_SHIFT && !(state&STATE_MASK_CTRL))
				rotation_angle*=pow(1.23,(double)zDelta/120.0);
			else if(state&STATE_MASK_ALT && !(state&STATE_MASK_CTRL) && !(state&STATE_MASK_SHIFT))
			{
				double hed=PointCloudScreen.getHalfEyeDistance();
				PointCloudScreen.setHalfEyeDistance(hed*pow(1.07,(double)zDelta/120.0));
				OnExpose(0);
			}
			else
			{
				factor*=pow(1.07,(double)zDelta/120.0);
				PointCloudScreen.setPerspective(distance,factor);
				OnExpose(0);
			}	
			break;
		}
	}
	return true;
}

void PathViewer::OnDestroyWindow()
{

}

bool PathViewer::selectEventTypes()
{
/*
	//#define TLC_REFRACTIVE_INDEX			0
	//#define TLC_TOTAL_ABSORPTION_COEFFICIENT	1
	//#define TLC_TOTAL_SCATTERING_COEFFICIENT	2
	//#define TLC_GROUND				2
	//#define TLC_EPSILON_SCATTERER			3	// generic scatterer
	//#define TLC_EPSILON_RAYLEIGH			3
	//#define TLC_EPSILON_ROTATIONAL_RAMAN		4
	//#define TLC_EPSILON_AEROSOL		 	5
	//
	// double wavelength=m_wavelength;
	struct GenericVariable_s GV[]=
	{
		{"refraction points :",					GV_boolean,enable_trace_event+TLC_REFRACTIVE_INDEX},
		{"absorption events :",					GV_boolean,enable_trace_event+TLC_TOTAL_ABSORPTION_COEFFICIENT},
		{"ground scatter events :",				GV_boolean,enable_trace_event+TLC_GROUND},
		{"rayleigh scatter events :",			GV_boolean,enable_trace_event+TLC_EPSILON_RAYLEIGH},
		{"rotational raman scatter events :",	GV_boolean,enable_trace_event+TLC_EPSILON_ROTATIONAL_RAMAN},
		{"aerosol 1 scatter events :",			GV_boolean,enable_trace_event+TLC_EPSILON_AEROSOL},
		{"aerosol 2 scatter events :",			GV_boolean,enable_trace_event+TLC_EPSILON_AEROSOL+1},
		{"aerosol 3 scatter events :",			GV_boolean,enable_trace_event+TLC_EPSILON_AEROSOL+2},
		{"aerosol 4 scatter events :",			GV_boolean,enable_trace_event+TLC_EPSILON_AEROSOL+3}
	};
	CParameterListAdjustDialog PLADialog;
	PLADialog.SetParameterListName("scatter event selection");
	if(!PLADialog.defineParameterList(GV,sizeof(GV)/sizeof(GV[0])))
		return;
	PLADialog.DoModal();
	PostMessage(WM_COMMAND,ID_DepictionReload);
*/
	return true;
}

int PathViewer::findIndexOfClosestPoint(struct ScreenPoint_s point, struct ScreenPoint_s *pScreenPoint)
{
	class Vector3D p0;
	struct ColoredPoint_s *cp=TraceEvents.getColoredPointBase();
	int i, n=TraceEvents.getNOPoints(), imin=0, x, y;
	double rmin=(double)max(PointCloudScreen.getXRes(), PointCloudScreen.getYRes()), r, dx, dy;
	rmin*=rmin;
	for(i=0;i<n;i++)
	{
		PointCloudScreen.toScreenPoint(cp[i].point,x,y);
		dx=x-point.x; dy=y-point.y;
		r=dx*dx+dy*dy;
		if(r<rmin)
		{
			rmin=r;
			imin=i;
		}
	}	
	if(pScreenPoint)
	{
		PointCloudScreen.toScreenPoint(cp[imin].point, x, y);
		pScreenPoint->x=x;
		pScreenPoint->y=y;
	}
	return imin;
}


void PathViewer::restorePerspective()
{
	Matrix3D m=Matrix3D(base_vector);
	resetBasisVectors();
	T=Vector3D(0.0,0.0,0.0);
	TransformPoints(m);
	TranslatePoints(T);
}

void PathViewer::ResetPerspective()
{
	resetBasisVectors();
	T=Vector3D(0.0,0.0,0.0);
	return;
}

void PathViewer::resetBasisVectors()
{
	for(int i=0;i<3;i++)
		base_vector[i]=Vector3D(i==0?1.0:0.0,i==1?1.0:0.0,i==2?1.0:0.0);
}

void PathViewer::TransformPoints(class Matrix3D& m)
{
	TraceEvents.transformPoints(m);
	SurfaceGridPoints.transformPoints(m);
	T*=m;
	for(int i=0;i<4;i++)
		base_vector[i]*=m;
}

void PathViewer::TranslatePoints(class Vector3D& delta_p)
{
	TraceEvents.translatePoints(delta_p);
	SurfaceGridPoints.translatePoints(delta_p);
	T+=delta_p;
}

/*
 *
void CPathViewerDlg::OnExportDepictionToEPS() 
{
	CFileDialog FileDialog(FALSE,"*.eps");
	if(FileDialog.DoModal()==IDOK)
	{
		char filename[4096];
		strcpy(filename, (char *)(LPCTSTR)FileDialog.GetPathName());
		class PointCollection AllPoints;
		if(!AllPoints.addPointCollection(&TraceEvents) || !AllPoints.addPointCollection(&SurfaceGridPoints))
			return;
		PointCloudScreen.exportToEPSFile(&AllPoints, filename);
		//PointCloudScreen.exportToEPSFile(&TraceEvents, filename);
	}
}

*/
