#ifndef _PathViewer_h
#define _PathViewer_h

#include "Y/YWindow.h"
#include "Y/YImage.h"

#include "../../Simulation.h"
#include "../TraceEventCollection.h"
#include "../SurfaceGridPointCollection.h"
#include "../RenderScreen.h"

struct ScreenPoint_s
{
	int x,y;
};

struct Marker_s
{
	int index;
	struct ScreenPoint_s p;
	class Vector3D point;
};

class PathViewer: public YWindow, public Simulation
{

	// point collections...
	class TraceEventCollection TraceEvents;
	class SurfaceGridPointCollection SurfaceGridPoints;
	class RenderScreen PointCloudScreen;

	// events
	int maximal_event_type_index;
	bool enable_trace_event[1000];
	bool selectEventTypes();

	// mouse
	bool update_size_display;
	bool left_mouse_button_down;
	bool middle_mouse_button_down;
	bool right_mouse_button_down;
	struct ScreenPoint_s mouse_point;
	int findIndexOfClosestPoint(struct ScreenPoint_s point, struct ScreenPoint_s *pScreenPoint=0);

	// operations
	double rotation_angle;
	class Vector3D T,base_vector[4];

	void resetBasisVectors();
	void ResetPerspective();
	void restorePerspective();
	void TranslatePoints(class Vector3D& delta_p);
	void TransformPoints(class Matrix3D& m);

	// markers
	struct Marker_s MarkerLeft, MarkerCenter, MarkerRight;
	void drawMarkers();
	void resetMarkers();

	// image
	void RenderClientWindow();
	YImage DrawingBoard;
	bool resizeScreen(int xres, int yres);
	int width, height;
	PathViewer(){}

	bool loadTrajectories(char *filename, FILE *logstream=stdout);
	bool loadScatteredTrajectories(FILE *logstream=stdout);
	bool loadUnscatteredTrajectories(FILE *logstream=stdout);
	char trajectory_filename[2047];
protected:
	virtual void OnCreateWindow();
	//virtual bool OnResize(XResizeRequestEvent *pXResizeRequestEvent);
	virtual bool OnConfigureNotify(XConfigureEvent *pXConfigureEvent);
	virtual bool OnExpose(XExposeEvent *pXExposeEvent);
	virtual bool OnMotion(XMotionEvent *pXMotionEvent);
	virtual bool OnKeyPress(XKeyPressedEvent *pXKeyPressedEvent);
	virtual bool OnButtonPress(XButtonPressedEvent *pXButtonPressedEvent);
	virtual bool OnButtonRelease(XButtonReleasedEvent *pXButtonReleasedEvent);
	virtual void OnDestroyWindow();


public:
	PathViewer(int argc, char *argv[]);
	//virtual bool processEvent(XEvent *pEvent);
};

#endif // _PathViewer_h

