#include "Y.h"
#include "YWindow.h"

Y::Y()
{
	no_windows=0;
	pYWindow=(class YWindow **)0;
	display=(Display *)0;
	openDisplay(NULL);
}

Y::Y(char *display_name)
{
	no_windows=0;
	pYWindow=(class YWindow **)0;
	display=(Display *)0;
	openDisplay(display_name);
}

Y::~Y()
{
	if(pYWindow)
	{
		delete [] pYWindow;
		pYWindow=(class YWindow **)0;
	}
	if(display)
	{
		XCloseDisplay(display);
		display=(Display *)0;
	}
}


int YErrorHandler(Display *pDisplay, XErrorEvent *pXErrorEvent)
{
/*
typedef struct {
	int type;
	Display *display;		// Display the event was read from
	unsigned long serial;		// serial number of failed request
	unsigned char error_code;	// error code of failed request
	unsigned char request_code;	// Major op-code of failed request
	unsigned char minor_code;	// Minor op-code of failed request
	XID resourceid;			// resource id
} XErrorEvent;

*/
	switch(pXErrorEvent->request_code)
	{
		case BadAccess:
			break;
		case BadAlloc:
			break;
		case BadAtom:
			break;
		case BadColor:
			break;
		case BadCursor:
			break;
		case BadDrawable:
			break;
		case BadFont:
			break;
		case BadGC:
			break;
		case BadIDChoice:
			break;
		case BadImplementation:
			break;
		case BadLength:
			break;
		case BadMatch:
			break;
		case BadName:
			break;
		case BadPixmap:
			break;
		case BadRequest:
			break;
		case BadValue:
			break;
		case BadWindow:
			break;
	}
	return 0;
	//http://tronche.com/gui/x/xlib/event-handling/protocol-errors/default-handlers.html#XErrorEvent
}


int YIOErrorHandler(Display *pDisplay)
{
	return 0;
}

bool Y::openDisplay(char *display_name)
{
	display=XOpenDisplay(display_name);
	if(!display)
	{
		  fprintf(stderr,"Y: cannot connect to X server %s\n", XDisplayName(display_name));
		  return false;
	}
	screen = DefaultScreen(display);
	display_width = DisplayWidth(display, screen);
	display_height = DisplayHeight(display, screen);
	//XSetIOErrorHandler(&YIOErrorHandler);
	//XSetErrorHandler(&YErrorHandler);
	return true;
}

bool Y::registerWindow(class YWindow *YWindowPtr)
{
	class YWindow **base=new class YWindow *[1+no_windows];
	if(!base)
	{
		fprintf(stderr,"file \"%s\", line %d: allocation error!\n", __FILE__, __LINE__);
		return false;
	}
	if(pYWindow)
	{
		for(int i=0;i<no_windows;i++)
			base[i]=pYWindow[i];
		delete [] pYWindow;
	}
	pYWindow=base;
	pYWindow[no_windows++]=YWindowPtr;
	return true;
}

void Y::closeWindows()
{
	int i, n=0;
	for(i=0;i<no_windows;i++)
		if(!pYWindow[i]->keepOpen())
			n++;
	if(n==no_windows)
	{
		delete [] pYWindow;
		pYWindow=(class YWindow **)0;
		no_windows=0;
	}
	else if(n>0)
	{
		int j=0;
		class YWindow **base=new class YWindow *[no_windows-n];
		for(i=0;i<no_windows;i++)
			if(pYWindow[i]->keepOpen())
				base[j++]=pYWindow[i];
		delete [] pYWindow;
		pYWindow=base;
		no_windows-=n;
	}
}

bool Y::processSystemEvents()
{
	int i;
	XEvent report;
	while (1)
	{
		XNextEvent(display, &report);
		for(i=0;i<no_windows;i++)
			if(pYWindow[i]->processEvent(&report))
				break;
		closeWindows();
		if(!no_windows)
			break;
	}
	return true;
}


