#include "YWindow.h"

YWindow::YWindow()
{
	created=false;
	gc_valid=false;
}

YWindow::~YWindow()
{
	if(gc_valid)
	{
		XFreeGC(pDisplay, gc);
		gc_valid=false;
	}
}

bool YWindow::createSimpleWindow(const char *window_name,\
	int x, int y, unsigned int width, unsigned int height, unsigned int border_width,\
	unsigned long border, unsigned long background)
{
	if(created)
	{
		fprintf(stderr,"window already created...\n");
		return false;
	}
	window = XCreateSimpleWindow(pDisplay,RootWindow(pDisplay,pY->getScreen()),x,y,width,height,border_width,border,background);
	XSelectInput(pDisplay, window,
		ExposureMask |
		KeyPressMask |
		ButtonPressMask |
		ButtonReleaseMask |
		PointerMotionMask |
		Button1MotionMask |
		Button2MotionMask |
		Button3MotionMask |
		Button4MotionMask |
		Button5MotionMask |
		StructureNotifyMask);

	SizeHints.flags = PPosition | PSize | PMinSize;
	SizeHints.x = x;
	SizeHints.y = y;
	SizeHints.width = width;
	SizeHints.height = height;
	SizeHints.min_width = width;
	SizeHints.min_height = height;
	setWindowTitle(window_name);
	XMapWindow(pDisplay, window);
	created=pY->registerWindow(this);
	if(created)
	{
		keep_open=true;
		if(gc_valid)
			XFreeGC(pDisplay, gc);
		gc=XCreateGC(pDisplay, window, 0, &GCvalues);
		gc_valid=true;
		OnCreateWindow();
		return true;
	}
	return false;
}

void YWindow::removeEventFromQueue(XEvent *pEvent)
{
	while (XCheckTypedEvent(pDisplay, pEvent->type , pEvent));
}

void YWindow::setWindowTitle(const char *title)
{
/*
	typedef struct
	{
		unsigned char *value;               // same as Property routines
		Atom encoding;                      // prop type
		int format;                         // prop data format: 8, 16, or 32
		unsigned long nitems;               // number of data items in value
	} XTextProperty;
*/
	strcpy(window_name, title);
	XTextProperty TextProperty;
	TextProperty.value=(unsigned char *)title;
	TextProperty.encoding=XCompoundTextStyle;
	TextProperty.format=8;
	TextProperty.nitems=strlen(title);
	XSetWMName(pDisplay, window, &TextProperty);
}

void YWindow::setClassName(const char *res_name, const char *res_class)
{
	XClassHint ClassHint;
	ClassHint.res_name=(char *)res_name;
	ClassHint.res_class=(char *)res_class;
	XSetClassHint(pDisplay, window, &ClassHint);
}

bool YWindow::processEvent(XEvent *pEvent)
{
	XAnyEvent *pXAnyEvent=(XAnyEvent *)pEvent;
	if(pXAnyEvent->window!=window)
		return false;
	switch(pEvent->type)
	{
		case Expose:
			//printf("Expose\n");
			return OnExpose(&(pEvent->xexpose));
		case ResizeRequest:
			return OnResize(&(pEvent->xresizerequest));
		case ConfigureNotify:
			//printf("ConfigureNotify\n");
			return OnConfigureNotify(&(pEvent->xconfigure));
		case ButtonRelease:
			//printf("ButtonRelease\n");
			return OnButtonRelease(&(pEvent->xbutton));
		case ButtonPress:
			//printf("ButtonPress\n");
			return OnButtonPress(&(pEvent->xbutton));
		case MotionNotify:
			//printf("MotionEvent\n");
			return OnMotion(&(pEvent->xmotion));
		case KeyPress:
			//printf("KeyPress\n");
			return OnKeyPress(&(pEvent->xkey));
		case DestroyNotify:
			//printf("DestroyNotify\n");
			keep_open=false;
			OnDestroyWindow();
			return true;
		default:;
	}
	return false;
}

